# -*- coding: utf-8 -*-
from lavda.analysis.da_clean.clean_interface import *


class AllTextSpam(RowInterface, CleanInterface):
    """
    电商垃圾的处理
    """
    def __init__(self, **params):
        super(AllTextSpam, self).__init__(CleanFlag.NO)

    def process(self, data, **params):
        if str(data) in ["转发微博", "轉發微博", "转发", "666", "好评"]:
            return CleanFlag.YES
        else:
            return CleanFlag.NO


if __name__ == '__main__':
    spam = AllTextSpam()
    print(spam.process("转发微博"))



# -*- coding: utf-8 -*-
# ----------------------------------------------
# purpose : 标签词表校验
# author : LGD
# create_time : 2021/2/22 14:31
# update_time : 2021/2/22 14:31
# copyright : Lavector
# ----------------------------------------------
import re
import pandas as pd
from openpyxl import load_workbook
import xlrd

from lavda.analysis.da_tag.label_dict import PandaLabelDictTool


class VerifyLabelDict(object):
    """
        词表校验
        1.词表，有隐藏sheet，有隐藏字段
        2.词表有空格（非关键词单元格）不能有空格，
        3.连续出现的竖线、单元格开头、结尾不能有坚线、特殊的竖线 如‘｜’就是错误的，比一般的‘|’宽
        :return:
    """

    def __init__(self, path):
        self.path = path

    @property
    def check_hide_sheet(self):
        """
        检查是否有隐藏sheet
        :return:
        """
        error_msg_list = list()
        check_code = 0
        check_msg = '有隐藏表'
        xb = pd.ExcelFile(self.path)
        s = xb.book._sheets
        for i in s:
            if i.sheet_state == 1:
                check_code = '有隐藏表'
                error_msg_list.append(['有隐藏表', ""])
                return error_msg_list

        return error_msg_list

    @property
    def check_hide_line(self):
        """
        检查是否有隐藏行或隐藏列
        :return:
        """
        error_msg_list = list()
        check_code = 0
        check_msg = '有隐藏行或列'
        wb = load_workbook(self.path)
        for sheet in wb:
            # name = re.findall(r'"(.*?)"', str(sheet))[0]  # sheet name
            for colLetter, colDimension in sheet.column_dimensions.items():
                if colDimension.hidden == True:
                    check_code = '有隐藏行或列'
                    error_msg_list.append(['有隐藏行或列', ''])
                    return error_msg_list

            for colLetter, colDimension in sheet.row_dimensions.items():
                if colDimension.hidden == True:
                    check_code = '有隐藏行或列'
                    error_msg_list.append(['有隐藏行或列', ''])
                    return error_msg_list

        return error_msg_list

    @property
    def check_null(self):
        error_msg_list = list()
        check_code = 0
        check_msg = '单元格错误-空单元格'
        excel = xlrd.open_workbook(self.path)
        for n in range(excel.nsheets):
            sheet = excel.sheet_by_index(n)
            sheet_name = excel._sheet_names[n]
            if '-' in sheet_name:
                num = int(sheet_name.split('-')[-1])-1
                df = pd.read_excel(self.path, sheet_name=sheet_name,header=None, nrows=num)
                for e in sheet.merged_cells:
                    rl, rh, cl, ch = e
                    base_value = sheet.cell_value(rl, cl)
                    if rl >= num:
                        error_msg_list.append(['有不合法的合并单元格', base_value])
                    if base_value == '':
                        error_msg_list.append(['合并单元格中有空单元格', ''])
                    df.iloc[rl:rh, cl:ch] = base_value
                for k in list(df.isnull().any()):
                    if k:
                        check_code = '有空单元格'
                        error_msg_list.append(['有空单元格', ''])
                        return error_msg_list

        return error_msg_list

    @staticmethod
    def check_keyword(keyword_str):
        error_msg_list = list()
        if "||" in str(keyword_str):
            error_msg_list.append(['有||在里面', keyword_str])
        if '｜' in str(keyword_str):
            error_msg_list.append(['有特殊的竖线(｜)在里面', keyword_str])
        if str(keyword_str)[0] == "|" or str(keyword_str[-1]) == "|":
            error_msg_list.append(['以|开头或结尾', keyword_str])
        for str_i in str(keyword_str).split('|'):
            if '+' in str_i and '-' in str_i:
                if not ('(' in str_i or ')' in str_i or '（' in str_i or '）' in str_i):
                    error_msg_list.append(['加减号必须用括号隔开', keyword_str])
        if '(' in keyword_str or ')' in keyword_str or '（' in keyword_str or '）' in keyword_str:
            veri = keyword_str.replace('）', ')').replace('（', '(')
            brackets_content = re.finditer(r'\((.*?)\)', veri)
            for e_item in brackets_content:
                e = e_item.group()
                e_start = e_item.start()
                e_end = e_start + len(e)
                left_margin_char = keyword_str[e_start-1] if e_start-1 >=0 else None
                right_margin_char = keyword_str[e_end] if e_end < len(keyword_str) else None
                if left_margin_char is not None and left_margin_char not in {"-", '+', '#', '@', '|'}:
                    error_msg_list.append(['括号左侧有不合法字符:'+str(left_margin_char), keyword_str])
                if right_margin_char is not None and right_margin_char not in {"-", '+', '#', '@', '|'}:
                    error_msg_list.append(['括号右侧有不合法字符:'+str(right_margin_char), keyword_str])

                if '-' in e:
                    error_msg_list.append(['括号内不能出现减号', keyword_str])
                if '|' in e:
                    error_msg_list.append(['括号内不能出现竖线', keyword_str])
                if len(e.split('+')) != 2:
                    error_msg_list.append(['括号里没有加号或加号超过两个', keyword_str])

        return error_msg_list

    @property
    def verification(self):
        """
        检查keyword
        :return:
        """
        error_msg_list = list()
        dict_tool = PandaLabelDictTool()

        mc = dict_tool.load_all_from_file(self.path)
        for mapping in mc.mappings:
            for keyword_str in list(mapping.id2keyword.values()):
                error_msg_list.extend(VerifyLabelDict.check_keyword(keyword_str))

        return error_msg_list

    def analyze(self):
        # 错误信息 list([错误类型， 错误内容]
        total_error_msg = list()
        total_error_msg.extend(self.check_hide_sheet)
        total_error_msg.extend(self.check_null)

        if len(total_error_msg) == 0:
            total_error_msg.extend(self.check_hide_line)
            total_error_msg.extend(self.verification)
        else:
            total_error_msg.extend(self.check_hide_line)

        return total_error_msg


if __name__ == '__main__':
    keyword_str = "精美|精致-(精致-女人)-精致|惊艳"
    msg = VerifyLabelDict.check_keyword(keyword_str)
    print(msg)





# -*- coding: utf-8 -*-
# ----------------------------------------------
# purpose: 提取用户类型 PGC/UGC/BGC
# author : 
# create_time : 2020/6/30 10:23
# update_time : 2020/6/30 10:23
# copyright : Lavector
# ----------------------------------------------
from lavda.backexcutor.analysis_interface import *
from lavda.util.mysql_util import LavwikiReader
import pandas as pd


class UserTag(RowInterface):
    def __init__(self, classify, default_value=None):
        super(UserTag, self).__init__(default_value)
        classify = classify.split('&')
        tag_df = LavwikiReader.get_as_df("lav2bgc_star")[
            ['classify', 'channel', 'userid', 'user_tag', 'brand']]

        tag_bgc_format = tag_df[(tag_df['classify'] == 'currency')]
        tag_pgc_format = tag_df[(tag_df['classify'] != 'personal')].loc[:, ('classify', 'channel', 'userid', 'user_tag')]
        if len(classify) > 0:
            tag_bgc_format = '''tag_df[(tag_df['classify'] == 'currency') '''
            tag_pgc_format = '''tag_df[(tag_df['classify'] != 'personal') '''
            for e_classify in classify:
                tag_bgc_format = tag_bgc_format + '''| (tag_df['classify'] == "{}") '''.format(e_classify)
                tag_pgc_format = tag_pgc_format + '''& (tag_df['classify'] != "{}") '''.format(e_classify)
            tag_bgc_format = tag_bgc_format + "]"
            tag_pgc_format = tag_pgc_format + "].loc[:,('classify', 'channel', 'userid', 'user_tag')]"
        tag_bgc_df = eval(tag_bgc_format)
        tag_pgc_df = eval(tag_pgc_format)

        # tag_bgc_df = tag_df[(tag_df['classify'] == classify) | (tag_df['classify'] == 'currency')]
        tag_star_df = tag_df[tag_df['classify'] == 'personal']
        # tag_pgc_df = tag_df[(tag_df['classify'] != 'personal') & (tag_df['classify'] != classify)].loc[:,
        #              ('classify', 'channel', 'userid', 'user_tag')]
        tag_pgc_df.loc[:, 'user_tag'] = 'pgc'
        # 下面这四行是为了去除重复的userID
        bgc_list = list(tag_bgc_df['userid'])
        tag_pgc_df['kk'] = tag_pgc_df.apply(lambda x: x['userid'] in bgc_list, axis=1)
        tag_pgc_df = tag_pgc_df[tag_pgc_df['kk'] == False]
        self.tag_dict = {}
        this_tag_df = pd.concat([tag_bgc_df, tag_star_df, tag_pgc_df], sort=False)
        for a, b in this_tag_df.groupby('channel'):
            b.index = b['userid']
            self.tag_dict[a] = b['user_tag'].to_dict()

        self.redbook_brand_list = []
        for e_classify in classify:
            redbook_brand_list = [i.lower() for i in list(
            tag_df[tag_df['channel'] == 'redbook'][tag_df[tag_df['channel'] == 'redbook']['classify'] == e_classify][
                'brand'])]
            self.redbook_brand_list.extend(redbook_brand_list)
        self.redbook_brand_list = list(set(self.redbook_brand_list))
        self.redbook_pgc_list = ['qq.com', '163.com', '126.com', 'icloud.com', 'htomail.com', 'xzwh.com',
                                 'yahoo.com', 'gmail.com', '邮箱', '美妆博主', '原创博主', '时尚博主', '店铺', '连锁店',
                                 '店主', 'b站', 'wb', '微博', 'ins', '同名', '干货', '分享', '直播', '搬运', '金冠薯']
        self.weibo_pgc_list = ['自媒体', '官方网站', '官方微博', '学者', '作家', '专家', '教授', '导师', '医师', '创始人', '主理人', '达人', '博主',
                               '主编', '编辑', '编剧', '漫画家', '撰稿人', '电竞选手', '游戏选手', '评论人', '影评人', '剧评人', '股评师', '股评人',
                               '优秀回答者', '顾问', '测评', '评测', '安利', '投资', 'vlog', '欢迎投稿', '答主', '问答', '原创', '头条文章作者',
                               '游戏视频自媒体', '医生', '美妆博主', '美食博主', '时尚博主', '摄影博主', '中国区总代理', '经销商']
        self.weibo_star_list = ['反黑站']
        self.tiktok_pgc_list = ['商业合作', '合作', '商务', '联系', '全网同名', '说明来意', '注明来意', '邮箱',
                                '导演', '制作人', '造型师', '专家', '导师', '医生', '博主', '创作者', '成员', '官方',
                                'qq.com', '163.com', '126.com', 'icloud.com', 'htomail.com', 'xzwh.com',
                                'yahoo.com', 'gmail.com', 'vx', 'wx', '微信', '酷狗', 'b站', 'q音', '网易云',
                                'vb', '微博', '微吧', '围脖', 'wb', 'weibo', ]

    def func_redbook(self, data, judge_text, user_tag='ugc'):
        # 认证
        authentication = data['authentication']
        if authentication == '蓝标':
            user_tag = 'pgc'
            if list(filter(lambda x: x in data['nickname'], self.redbook_brand_list)):
                user_tag = 'bgc'
                return user_tag
            return user_tag
        if authentication == '红标':
            user_tag = 'pgc'
            return user_tag
        # 粉丝
        if str(data['fans']).isnumeric():
            ss = int(data['fans'])
            if ss <= 1000 and user_tag != 'pgc':
                user_tag = 'ugc'
            else:
                user_tag = 'pgc'
                return user_tag
        # 简介、昵称、用户标签
        if list(filter(lambda x: x in judge_text, self.redbook_pgc_list)):
            user_tag = 'pgc'
        return user_tag

    def func_tiktok(self, data, judge_text, user_tag='ugc'):
        # 粉丝
        if str(data['fans']).isnumeric():
            ss = int(data['fans'])
            if ss <= 5000 and user_tag != 'pgc':
                user_tag = 'ugc'
            else:
                user_tag = 'pgc'
                return user_tag
        # 认证
        if data['authentication'] == '红标' or data['authentication'] == '蓝标':
            user_tag = 'pgc'
            return user_tag
        # 简介、昵称、用户标签
        if list(filter(lambda x: x in judge_text, self.tiktok_pgc_list)):
            user_tag = 'pgc'

        return user_tag

    def func_weibo(self, data, judge_text, user_tag='ugc'):
        # 粉丝
        if str(data['fans']).isnumeric():
            ss = int(data['fans'])
            if ss <= 3000 and user_tag != 'pgc':
                user_tag = 'ugc'
            else:
                user_tag = 'pgc'
                return user_tag
        # 认证
        if data['authentication'] == '黄标' or data['authentication'] == '蓝标':
            user_tag = 'pgc'
            return user_tag
        # 简介、昵称、用户标签
        if list(filter(lambda x: x in judge_text, self.weibo_star_list)):
            user_tag = 'star'
            return user_tag
        if list(filter(lambda x: x in judge_text, self.weibo_pgc_list)):
            user_tag = 'pgc'

        return user_tag

    def distribute(self, data, **params):
        '''
        分布处理三个平台的数据
        :param data: 字典类型一行数据
        :param params:
        :return:
        '''
        judge_column_name_list = ['fans', 'authentication', 'introduce', 'user_label', 'nickname']
        for column_name in judge_column_name_list:
            if column_name not in data:
                data[column_name] = ''
        judge_text = '.'.join([str(data[i]) for i in judge_column_name_list[1:] if data[i]]).lower()
        channel_dict = {'redbook': self.func_redbook, 'weibo': self.func_weibo, 'tiktok': self.func_tiktok}
        return channel_dict[data['channel']](data, judge_text)

    def process(self, data, **params):
        """
        判断用户类别
        """
        user_tag = 'ugc'  # 默认为ugc
        if data['channel'] in self.tag_dict and data['userid'] in self.tag_dict[data['channel']]:
            user_tag = self.tag_dict[data['channel']][str(data['userid'])]
        if data['channel'] in ['redbook', 'weibo', 'tiktok'] and user_tag not in ['bgc', 'star']:
            user_tag = self.distribute(data)
        # TODO 默认给定ugc
        return user_tag


if __name__ == '__main__':


    # classify = ['美妆', '个护']
    classify = ['皮肤美容大健康', '保健品']
    user_tag = UserTag(classify)
    # data = {'channel': 'weibo', 'userid': '1016215307'} # 皮肤美容大健康 保健品 bgc

    df = pd.read_excel('data.xlsx')


    def func(line):
        dic = line.to_dict()
        return user_tag.process(dic)
    df['user_tag'] = df.apply(func, axis=1)
    split_count = df.shape[0] // 1000000 + 1
    for i in range(split_count):
        writer = pd.ExcelWriter('data_结果文件_{}.xlsx'.format(i), options={'strings_to_urls': False}, engine='xlsxwriter')
        df[i * 1000000:(i + 1) * 1000000].to_excel(writer, sheet_name=str(i), header=True, index=False,
                                                   encoding="utf_8_sig")
        print(i)
        writer.save()



# -*- coding: utf-8 -*-
# ----------------------------------------------
# purpose : 
# author : LGD
# create_time : 2021/2/26 10:51
# update_time : 2021/2/26 10:51
# copyright : Lavector
# ----------------------------------------------
import re
import pandas as pd
import string

from lavda.util.text_util import DBC2SBC

gl_letter_sets = set(string.ascii_letters)

class LabelUtil(object):
    RE_USER_TAT = re.compile('@[\S]+') #re.compile('@.*? ')
    RE_TOPIC = re.compile('#.*?#')
    RE_EMOTION = re.compile("["
                               u"\U0001F600-\U0001F64F"  # emoticons
                               u"\U0001F300-\U0001F5FF"  # symbols & pictographs
                               u"\U0001F680-\U0001F6FF"  # transport & map symbols
                               u"\U0001F1E0-\U0001F1FF"  # flags (iOS)
                               "]+", flags=re.UNICODE)

    @staticmethod
    def clean_text(content):
        if pd.isna(content):
            return ""
        text = str(content)
        text = LabelUtil.remove_topic(text)

        text = LabelUtil.RE_EMOTION.sub(r'', text)
        line = re.sub(r"[\n]+", "。", text.strip())
        # line = re.sub(r"[!！?？#,，@… ~～。.`、'‘]+", " ", line)
        # line = re.sub(r"[#]+", " ", line)
        line = re.sub(r"( ){2,}", "", line)
        line = LabelUtil.format_eng_text(line)

        line = line.lower().strip()
        line = DBC2SBC(line)

        return line

    @staticmethod
    def clean_dict_cell(content):
        """
        标签词典单元格清理方法
        :param content:
        :return:
        """
        if pd.isna(content):
            return ""
        line = str(content).strip()
        line = LabelUtil.format_eng_text(line)

        line = line.replace(" ", "")
        line = line.lower()

        line = DBC2SBC(line)

        return line

    @staticmethod
    def format_eng_text(content):
        """
        标准化英文数据，英文名字可能通过空格、引号等进行分隔处理，这个很不方便，因此要格式化
        :param content:
        :return:
        """
        line = content
        while 1:
            result = re.sub(r"(.*[A-Za-z])( |')+([A-Za-z].*)", r'\1_\3', line)
            if result == line:
                break
            line = result

        return line

    @staticmethod
    def remove_topic(content, user_max_len=16):
        """
        去除文本中的话题、用户名等
        :param content: string, 输入文本
        :param user_max_len: int,由于'@用户名'使用不规范，后面可能没有空格分隔，因此可以设置一个最大长度，超过此长度的不删除
        :return:
        """
        if '@' in content:
            # 正常的@用户
            li = LabelUtil.RE_USER_TAT.findall(content)
            for i in li:
                if user_max_len is not None:
                    if len(i) > user_max_len:
                        i = i[:user_max_len]

                content = content.replace(i, '')
        if '#' in content:
            content = LabelUtil.RE_TOPIC.sub("", content)

        return content


def run_one():
    data = "我肋中国#123#lsdfld,@lkl苈轸ewewrwer w "
    result = LabelUtil.clean_text(data)
    print(result)


def run_format_en():
    line = "Nature Way'lei佳思敏"
    while 1:
        result = re.sub(r"(.*[A-Za-z])( |')+([A-Za-z].*)", r'\1_\3', line)
        if result == line:
            break
        line = result

    print(line)


if __name__ == '__main__':
    run_format_en()


# -*- coding: utf-8 -*-
# ----------------------------------------------
# purpose : 创建数据id
# author : 
# create_time : 2020/7/3 13:28
# update_time : 2020/7/3 13:28
# copyright : Lavector
# ----------------------------------------------
import uuid

from lavda.backexcutor.analysis_interface import RowInterface


class CreateDataIndex(RowInterface):
    def __init__(self):
        super(CreateDataIndex, self).__init__(None)

    def process(self, data=None, **params):
        return str(uuid.uuid1()).replace("-", "")



# -*- coding: utf-8 -*-
# ----------------------------------------------
# purpose : 配置加载/解析功能
# author : 
# create_time : 2020/6/29 16:47
# update_time : 2020/6/29 16:47
# copyright : Lavector
# ----------------------------------------------
import xml.etree.ElementTree as ET


class ConfigUtil(object):
    @staticmethod
    def _clean_string(data, default_value=""):
        if data is None:
            return default_value
        return data.strip()

    @staticmethod
    def get_flow_config(file_path):
        """
        解析任务流配置
        :param file_path: 配置文件名，不带后缀
        :return:
        """
        tree = ET.parse(file_path)
        root_node = tree.getroot()
        config_list = list()

        for section_child in root_node:
            if len(section_child) <= 0:
                continue

            one_config_dict = dict()

            for item_node in section_child:
                item_key = ConfigUtil._clean_string(item_node.attrib[u'name'])
                if len(item_node) > 0:
                    sub_dict = dict()
                    one_config_dict[item_key] = sub_dict
                    for sub_item_node in item_node:
                        sub_item_key = ConfigUtil._clean_string(sub_item_node.attrib[u'name'])
                        sub_item_value = ConfigUtil._clean_string(sub_item_node.text)
                        sub_dict[sub_item_key] = sub_item_value
                else:
                    item_value = ConfigUtil._clean_string(item_node.text)
                    one_config_dict[item_key] = item_value

            if one_config_dict:
                config_list.append(one_config_dict)

        return config_list


if __name__ == '__main__':
    pass


# -*- coding: utf-8 -*-
# ----------------------------------------------
# purpose : 
# author : 
# create_time : 2020/7/2 16:07
# update_time : 2020/7/2 16:07
# copyright : Lavector
# ----------------------------------------------
import re

from lavda.analysis.da_filter.filter_interface import RowFilterInterface


class ColumnNotMatchFilter(RowFilterInterface):
    def __init__(self, input_data):
        tmp_list = input_data.split("=")
        t_key = tmp_list[0]
        t_value_str = tmp_list[1]
        t_value_list = re.split('[,，]', t_value_str)
        t_value_list = [item.strip() for item in t_value_list if item.strip()]

        self.name = t_key
        self.values = set(t_value_list)

    def process(self, data, **params):
        line, _ = data

        if line[self.name] not in self.values:
            return True

        return False



# -*- coding:utf-8 -*-
"""
文件相关操作
"""
from __future__ import print_function
from __future__ import absolute_import

import os
import shutil
import json
import pickle
import codecs
import pandas as pd
# import csv


class FileTool(object):
    @staticmethod
    def get_parent_dir(entire_path):
        return os.path.dirname(entire_path)

    @staticmethod
    def get_base_name(entire_path):
        """
        提取完整路径的文件名或目录名
        :param entire_path:
        :return:
        """
        return os.path.basename(entire_path)

    @staticmethod
    def rm_file(path):
        """
        删除文件或目录
        :param path:
        :return:
        """
        if not os.path.exists(path):
            return
        if os.path.isfile(path):
            os.remove(path)
        elif os.path.isdir(path):
            # os.rmdir(path)
            shutil.rmtree(path)

    @staticmethod
    def make_dir(path):
        """
        创建目录，可以递归
        :param path:
        :return:
        """
        if os.path.exists(path):
            return
        os.makedirs(path)

    @staticmethod
    def is_spam_file_name(file_name):
        """

        :param file_name: string,是否为垃圾文件名，不是完整路径，只是文件名
        :return:
        """
        if file_name.startswith(u'.') or file_name.startswith(u'~'):
            return True

        return False

    @staticmethod
    def get_all_file(dir_name, suffix=None, filter_file_name=True, recursion=True):
        """
        从文件夹或目录获取全部文件名(完整路径),不包含子目录
        :param dir_name:
        :param suffix: string or list 后缀名，如".txt"
        :param filter_file_name: bool,是否过虑隐形文件等
        :param recursion: bool,是否递归遍历
        :return: list(string)
        """
        suffix_list = None
        if suffix:
            if isinstance(suffix, (list, tuple, set)):
                suffix_list = suffix
            else:
                suffix_list = [suffix]

        fn_list = []
        for root, dirs, fns in os.walk(dir_name):
            if not recursion and root != dir_name:
                continue
            for fn in fns:
                # 校验后缀
                if suffix_list:
                    satisfy_suffix = False
                    for sub_suffix in suffix_list:
                        if fn.endswith(sub_suffix):
                            satisfy_suffix = True
                            break
                    if not satisfy_suffix:
                        continue
                # 过虑文件名
                if filter_file_name:
                    if FileTool.is_spam_file_name(fn):
                        continue
                temp_fn = os.path.join(root, fn)
                fn_list.append(temp_fn)

        return fn_list

    @staticmethod
    def get_file_path(input_path, suffix=None, recursion=False):
        """
        在指定目录获取指定文件名或以此文件名开头的文件的路径
        file_name 为空时，获取所有文件路径
        :param input_path: 目录名/文件名
        :param suffix: 文件后缀,输入为目录时有效
        :param recursion: 是否递归遍历,输入为目录时有效
        :return:
        """
        if os.path.isdir(input_path):
            all_file_path = FileTool.get_all_file(input_path, suffix=suffix, recursion=recursion)
            return all_file_path
        elif os.path.isfile(input_path):
            return [input_path]
        else:
            parent_file_path, temp_file_name = os.path.split(input_path)
            if not os.path.isdir(parent_file_path):
                raise Exception("invalid input = " + input_path)
            file_name_no_suffix, suffix = os.path.splitext(temp_file_name)

            # file_name_no_suffix, suffix = os.path.splitext(temp_file_name)
            if not suffix:
                return list()
            all_file_path = FileTool.get_all_file(parent_file_path, suffix=suffix, recursion=recursion)

            all_file_path = [item for item in all_file_path if os.path.isfile(item)]
            all_file_name_dict = {FileTool.get_base_name(item): item for item in all_file_path}

            # 根据前缀匹配
            result_list = list()
            for name in all_file_name_dict:
                tmp_name = name.replace(suffix, "")
                tmp_name = tmp_name.replace(file_name_no_suffix, "")
                tmp_name = tmp_name.replace("_", "")
                if tmp_name.isdigit():
                    result_list.append(all_file_name_dict[name])
            return result_list

    @staticmethod
    def get_all_dir(dir_name):
        """
        从文件夹或目录获取全部子目录(完整路径),不包含子目录
        :param dir_name:
        :param suffix: 后缀名，如".txt"
        :return: list(string)
        """
        fn_list = []
        for root, dirs, fns in os.walk(dir_name):
            for d in dirs:
                temp_fn = os.path.join(root, d)
                fn_list.append(temp_fn)

        return fn_list

    @staticmethod
    def write(file_path, data, binary=False):
        """
        保存数据到文件
        :param file_path: 文件名
        :param data: 任意类型
        :param binary: 是否保存为二进制文件
        :return:
        """
        option = 'wb' if binary else 'w'
        with open(file_path, option) as fp:
            pickle.dump(data, fp)

    @staticmethod
    def read(file_path, binary=False):
        """
        加载模型
        :param file_path: 模型文件
        :param binary: 是否为二进制文件
        :return:
        """
        option = 'rb' if binary else 'r'
        with open(file_path, option) as fp:
            data = pickle.load(fp)

        return data

    @staticmethod
    def write_json(file_path, data):
        """
        保存页面信息
        :param file_path:
        :param data:
        :return:
        """
        path = open(file_path, u'w')
        json.dump(data, path)

    @staticmethod
    def read_json(file_path):
        """
        读取json数据
        :param file_path:
        :return:
        """
        if os.path.exists(file_path):
            path = open(file_path, 'r')
            return json.load(path)
        return None

    @staticmethod
    def read_by_line(file_path):
        """
        按行读取数据
        :param file_path:
        :return:
        """
        data_list = list()
        with codecs.open(file_path, 'r', encoding="utf-8") as fp:
            for line in fp:
                line = line.strip()
                if len(line) > 0:
                    data_list.append(line)

        return data_list

    @staticmethod
    def write_by_line(file_path, data):
        """
        按行读取数据
        :param file_path:
        :param data:
        :return:
        """
        data_len = len(data)  #
        with codecs.open(file_path, 'w', encoding="utf-8") as fp:
            for i, item in enumerate(data):
                fp.write(item)
                if i < data_len - 1:
                    fp.write(u"\n")

    @staticmethod
    def write_str(file_path, data):
        with codecs.open(file_path, 'w', encoding="utf-8") as fp:
            fp.write(data)

    @staticmethod
    def read_str(file_path):
        data = u""
        with codecs.open(file_path, 'r', encoding="utf-8") as fp:
            for line in fp:
                data += line

        return data

    @staticmethod
    def read_by_delimiter(file_path, delimiter):
        """
        读取通过分隔符分开的数据，每行一条，且不能重复
        如nam=davic
        :param file_path:
        :param delimiter:
        :return:
        """
        data_dict = dict()
        data = u""
        with codecs.open(file_path, 'r', encoding="utf-8") as fp:
            for line in fp:
                items = line.split(delimiter)
                if items is None:
                    continue
                if len(items) != 2:
                    continue
                key = items[0].strip()
                value = items[1].strip()
                data_dict[key] = value

        return data_dict

    @staticmethod
    def read_csv(file_path, has_header=False, user_columns=None, real_columns=None):
        # file_data = open(file_path)
        if has_header:
            # 默认使用第一行 encoding="utf8",
            excel_data = pd.read_csv(file_path)
        else:
            excel_data = pd.read_csv(file_path, header=None)
        if user_columns:
            excel_data.columns = user_columns
        if real_columns is not None:
            cols = excel_data.columns
            for key in cols:
                real_columns.append(key)
        for index, row in excel_data.iterrows():
            yield index, row

    @staticmethod
    def read_excel(file_path, sheet_name=None, has_header=False, user_columns=None, real_columns=None):
        """
        读取excel数据，有表头时(即has_header为True)，会跳过第一行，直接返回数据，不会返回表头信息
        :param file_path:
        :param sheet_name:  sheet
        :param has_header:  是否有列名, True时：
        :param user_columns: 是否使用自定义列名
        :param real_columns: 实际列名， list ，输出数据
        :return:
        """
        file_data = open(file_path)
        if has_header:
            # 默认使用第一行
            excel_data = pd.read_excel(file_data, sheet_name=sheet_name)
        else:
            excel_data = pd.read_excel(file_data, sheet_name=sheet_name, header=None)
        if user_columns:
            excel_data.columns = user_columns
        if real_columns is not None:
            cols = excel_data.columns
            for key in cols:
                real_columns.append(key)
        for index, row in excel_data.iterrows():
            yield index, row

    @staticmethod
    def write_to_excel(file_path, data_list, columns, sheet=None, index=False):
        """

        :param file_path: string
        :param data_list: list(list)
        :param columns: list
        :param sheet: string
        :param index: bool ，是否添加左侧第一列作为索引
        :return:
        """
        df = pd.DataFrame(data_list, columns=columns)
        excel_writer = pd.ExcelWriter(file_path)

        df.to_excel(excel_writer=excel_writer, sheet_name=sheet, index=index)
        excel_writer.save()


if __name__ == '__main__':
    result = FileTool.get_file_path("/Users/yuantian/Downloads/PyTorch中文手册/README.md")
    print(result)
    print(len(result))


# -*- coding: utf-8 -*-
# ----------------------------------------------
# purpose : 
# author : 
# create_time : 2020/7/1 16:19
# update_time : 2020/7/1 16:19
# copyright : Lavector
# ----------------------------------------------
import pandas as pd
import collections
import os
import xlrd
import pickle
import contextlib


class PandaLabelDictTool(object):
    def check_file(self, input_file_path, file_suffix):
        parent_file_path, temp_file_name = os.path.split(input_file_path)
        file_name, extension = os.path.splitext(temp_file_name)
        extension = extension.lower()
        extension = extension.replace(".", "")
        if extension in file_suffix:
            return True
        else:
            raise Exception("unsupport file extension = " + extension)

    def trans(self, total_data_list):
        """
        二维数据置换
        :param total_data_list:
        :return:
        """
        old_max_rows = len(total_data_list)
        old_max_column = 0
        for line in total_data_list:
            current_column = len(line)
            if current_column > old_max_column:
                old_max_column = current_column

        result_data_list = list()
        for i in range(old_max_column):
            sub_data_list = list()
            for j in range(old_max_rows):
                if i < len(total_data_list[j]):
                    sub_data_list.append(total_data_list[j][i])
                else:
                    # TODO 空值填充
                    sub_data_list.append(None)
            result_data_list.append(sub_data_list)

        return result_data_list

    def is_none_value(self, cell_value):
        if pd.isna(cell_value):
            return True
        tmp_cell_value = str(cell_value)
        if not tmp_cell_value:
            return True

        return False

    def drop_none_row(self, total_data_list):
        """
        删除全为None 或空值的行
        :param total_data_list:
        :return:
        """
        result_data_list = list()
        for line in total_data_list:
            is_delete = True
            for cell_value in line:
                if not self.is_none_value(cell_value):
                    is_delete = False
                    break
            if not is_delete:
                result_data_list.append(line)

        return result_data_list

    def drop_none_column(self, total_data_list):
        """
        删除全为None 或空值的行
        :param total_data_list:
        :return:
        """
        delete_column_index = set()
        column_data_list = self.trans(total_data_list)
        for i, line in enumerate(column_data_list):
            is_delete = True
            for cell_value in line:
                if not self.is_none_value(cell_value):
                    is_delete = False
                    break
            if is_delete:
                delete_column_index.add(i)

        result_data_list = list()
        for line in total_data_list:
            sub_list = list()
            for j, cell_value in enumerate(line):
                if j not in delete_column_index:
                    sub_list.append(cell_value)
            result_data_list.append(sub_list)

        return result_data_list

    def check_has_non_value(self, total_data_list):
        for i, line in enumerate(total_data_list):
            for j, cell_value in enumerate(line):
                if self.is_none_value(cell_value):
                    raise Exception("has non value" + ", line=" + str(i) + ", col=" + str(j))

    def drop_end_non_value(self, total_data_list, total_label_level):
        """
        除最后一级标签外， 删除末尾的连续 空值
        :param total_data_list:
        :return:
        """
        result_data = list()
        for i, line in enumerate(total_data_list):
            label_level = i + 1
            if label_level < total_label_level:
                del_list = list()
                data_index = len(line) - 1
                while data_index >= 0:
                    item = line[data_index]
                    if self.is_none_value(item):
                        del_list.append(data_index)
                    else:
                        break
                    data_index -= 1

                if del_list:
                    sub_list = line[0: del_list[len(del_list) - 1]]
                else:
                    sub_list = line
            else:
                sub_list = line
            result_data.append(sub_list)

        return result_data

    def load_tree_dict(self, input_file_path,
                       sheet_name=None,
                       tree_format=0,
                       header=False,
                       total_label_level=5,
                       label_name=None,
                       split_text=False):
        """

        :param input_file_path: string, 输入文件路径
        :param sheet_name: string, sheet name
        :param tree_format: int, 格式，0：竖形树， 1：横着的树
        :param header: bool, 是否有表头，默认不保留表头信息
        :param total_label_level: int, 标签词典级别
        :param label_name: string, 标签词典级别
        :param split_text: bool, 最后一级是否需要拆分文本
        :return:
        """
        # 1.校验输入文件
        file_suffix = ['xlsx', 'xls']
        self.check_file(input_file_path, file_suffix)

        # 2.处理合并单元格
        excel_merge = LabelExcelMerge()
        dict_array = excel_merge.process(input_file_path, sheet_name=sheet_name)

        # 检查是否有表头
        # TODO 默认不需要表头数据
        if header:
            # 需要去掉表头数据
            dict_array = dict_array[1:]

        # 3.横排的要做转置
        if tree_format == 1:
            # 矩阵转换，横排的要做转换
            dict_array = self.trans(dict_array)

        # 4. 清理
        # 除最后一级外，删除所有行末尾的空值
        dict_array = self.drop_end_non_value(dict_array, total_label_level)
        dict_array = self.drop_none_row(dict_array)
        dict_array = self.drop_none_column(dict_array)

        # 除最后一级外，没能有空值
        if total_label_level > 1:
            self.check_has_non_value(dict_array[0: total_label_level - 1])

        # 以第一行为标准确定完整的列数
        max_column_count = len(dict_array[0])

        # 6. 加载词典信息
        # 全局信息
        cell_pos_dict = dict()
        # 记录最后一级以外的父节点信息
        cell_pos_list = list()

        label_id_dict = dict()
        total_label_id_count = 0

        for i, line in enumerate(dict_array):
            current_level = i + 1
            if current_level > total_label_level:
                current_level = total_label_level
            parent_level = current_level - 1

            sub_pos_list = list()
            cell_pos_list.append(sub_pos_list)
            for j, cell_value in enumerate(line):
                # 超过最大列的不处理
                if j >= max_column_count:
                    continue

                if pd.isna(cell_value):
                    sub_pos_list.append(None)
                    continue
                cell_value = str(cell_value).strip()
                if not cell_value:
                    sub_pos_list.append(None)
                    continue

                if current_level == 1:
                    parent_label_id = -1
                    parent_label_item = None
                else:
                    parent_label_item = cell_pos_list[parent_level - 1][j]
                    if parent_label_item is None:
                        raise Exception("parent is None: line=" + str(i) + ", row" + str(j))
                    parent_label_id = parent_label_item['id']

                # TODO 只对最后一级作拆分
                if current_level == total_label_level and split_text:
                    sub_cell_value_list = cell_value.split("|")
                    sub_cell_value_list = [sub_value.strip() for sub_value in sub_cell_value_list if sub_value.strip()]

                    for sub_cell_value in sub_cell_value_list:
                        pos_value = str(parent_label_id) + "-" + str(current_level) + "-" + sub_cell_value
                        if pos_value in cell_pos_dict:
                            sub_pos_list.append(cell_pos_dict[pos_value])
                            continue

                        new_label_item = dict()
                        new_label_item['id'] = total_label_id_count
                        new_label_item['level'] = current_level
                        new_label_item['parent'] = parent_label_id
                        new_label_item['text'] = sub_cell_value
                        cell_pos_dict[pos_value] = new_label_item
                        sub_pos_list.append(new_label_item)

                        label_id_dict[new_label_item['id']] = new_label_item

                        total_label_id_count += 1

                else:
                    pos_value = str(parent_label_id) + "-" + str(current_level) + "-" + cell_value
                    if pos_value in cell_pos_dict:
                        sub_pos_list.append(cell_pos_dict[pos_value])
                        continue

                    new_label_item = dict()
                    new_label_item['id'] = total_label_id_count
                    new_label_item['level'] = current_level
                    new_label_item['parent'] = parent_label_id
                    new_label_item['text'] = cell_value
                    cell_pos_dict[pos_value] = new_label_item
                    sub_pos_list.append(new_label_item)
                    label_id_dict[new_label_item['id']] = new_label_item

                    total_label_id_count += 1

        label_dict_object = TreeLabelDict(label_id_dict, total_label_level, label_name)

        return label_dict_object


class LabelDictInterface(object):
    def set_data_column(self, data_column):
        """

        :param data_column: list   标签所对应的数据列名
        :return:
        """
        pass

    def get_data_column(self):
        pass

    def get_label_name(self):
        """
        标签名称
        :return:
        """
        pass

    def set_label_name(self, label_name):
        pass

    def get_label_id_dict(self):
        pass

    def get_item_by_id(self, label_id):
        pass

    def get_label_text_by_id(self, label_id):
        pass

    def get_last_labels_item(self):
        """
        获取最后一级label 信息,即实际用于打标签的内容
        :return:
        """
        pass

    def get_parent_label_text(self, label_id, include_me=True):
        pass

    def get_label_detail_column(self, include_label_id=True):
        """
        标签详细信息列名
        :param include_label_id:  是否包含最后一级label id
        :return:
        """

    def get_label_detail(self, label_id, include_label_id=True):
        """
        标签详细信息
        :param label_id:
        :param include_label_id:
        :return:
        """
        pass

    def get_column_prefix(self):
        """
        列名前缀，用于作为统计汇总使用
        :return:
        """
        return "la_"

    def print_all(self):
        pass

    def print_detail(self):
        pass

    def get_processor(self, processor):
        pass

    def set_processor(self):
        pass


class TreeLabelDict(LabelDictInterface):
    def __init__(self, label_id_dict, level_count, label_name=None):
        self.label_id_dict = label_id_dict
        self.level_count = level_count
        self.label_name = label_name

        # TODO 用于打标签的数据列名,暂时放在这里吧
        self.data_column = None
        # TODO 标签处理类，暂时放这里吧
        self.processor = None

    def set_processor(self, processor):
        self.processor = processor

    def get_processor(self):
        return self.processor

    def set_data_column(self, data_column):
        self.data_column = data_column

    def get_data_column(self):
        return self.data_column

    def get_label_name(self):
        return self.label_name

    def set_label_name(self, label_name):
        self.label_name = label_name

    def get_label_id_dict(self):
        return self.label_id_dict

    def get_level_count(self):
        return self.level_count

    def get_item_by_id(self, label_id):
        return self.label_id_dict[label_id]

    def get_label_text_by_id(self, label_id):
        return self.label_id_dict[label_id]['text']

    def get_parent_by_level(self, current_label_id, parent_level):
        """
        查找相应级别的父类标签
        :param current_label_id:
        :param parent_level:
        :return:
        """
        parent_id = self.label_id_dict[current_label_id]['parent']
        while parent_id >= 0:
            parent_item = self.label_id_dict[parent_id]
            if parent_item['level'] == parent_level:
                return parent_item
            parent_id = parent_item['parent']

        return None

    def get_last_labels_item(self):
        """
        获取最后一级label 信息,即实际用于打标签的内容
        :return:
        """
        label_item_list = list()
        for label_id in self.label_id_dict:
            label_item = self.label_id_dict[label_id]
            if label_item['level'] == self.level_count:
                label_item_list.append(label_item)

        return label_item_list

    def get_label_detail_column(self, include_label_id=True):
        column_list = list()
        label_level = self.get_level_count()
        for i in range(label_level):
            column_list.append(self.get_column_prefix() + self.label_name + "_" + str(i + 1))
        if include_label_id:
            column_list.append(self.get_column_prefix() + self.label_name + "_" + "id")

        return column_list

    def get_label_detail(self, label_id, include_label_id=True):
        detail_list = self.get_all_parent_label_text(label_id)
        if include_label_id:
            detail_list.append(label_id)

        return detail_list

    def get_all_parent_label_text(self, current_label_id, include_me=True):
        parent_list = list()
        parent_id = self.label_id_dict[current_label_id]['parent']
        while parent_id >= 0:
            parent_list.insert(0, self.label_id_dict[parent_id]['text'])
            parent_id = self.label_id_dict[parent_id]['parent']
        if include_me:
            parent_list.append(self.get_item_by_id(current_label_id)['text'])

        return parent_list

    def print_all(self):
        level_count_dict = {level + 1: 0 for level in range(self.level_count)}
        for label_id in self.label_id_dict:
            level_count_dict[self.label_id_dict[label_id]['level']] += 1
        print(level_count_dict)

    def print_detail(self):
        for label_id in self.label_id_dict:
            print(self.label_id_dict[label_id])

    def get_dict(self):
        return self.label_id_dict


class LabelExcelMerge(object):
    def process(self, input_file_path, sheet_name=None):
        workbook = xlrd.open_workbook(input_file_path)

        # 获取所有sheet
        # print('打印所有sheet:', workbook.sheet_names())
        if not sheet_name:
            target_sheet = workbook.sheet_by_index(0)  # sheet索引从0开始
        else:
            target_sheet = workbook.sheet_by_name(sheet_name)
        rows_num = target_sheet.nrows
        cols_num = target_sheet.ncols

        # print(target_sheet.merged_cells)
        total_data_list = list()
        for r in range(rows_num):
            line_data_list = list()
            for c in range(cols_num):
                cell_value = target_sheet.row_values(r)[c]
                # print('第%d行第%d列的值：[%s]' % (r, c, sheet2.row_values(r)[c]))
                if cell_value is None or cell_value == '':
                    cell_value = (self.get_merged_cells_value(target_sheet, r, c))
                line_data_list.append(cell_value)

            total_data_list.append(line_data_list)

        return total_data_list

    def get_merged_cells(self, sheet):
        """
        获取所有的合并单元格，格式如下：
        [(4, 5, 2, 4), (5, 6, 2, 4), (1, 4, 3, 4)]
        (4, 5, 2, 4) 的含义为：行 从下标4开始，到下标5（不包含）  列 从下标2开始，到下标4（不包含），为合并单元格
        :param sheet:
        :return:
        """
        return sheet.merged_cells

    def get_merged_cells_value(self, sheet, row_index, col_index):
        """
        先判断给定的单元格，是否属于合并单元格；
        如果是合并单元格，就返回合并单元格的内容
        :return:
        """
        merged = self.get_merged_cells(sheet)
        for rlow, rhigh, clow, chigh in merged:
            if row_index >= rlow and row_index < rhigh:
                if col_index >= clow and col_index < chigh:
                    cell_value = sheet.cell_value(rlow, clow)
                    # print('该单元格[%d,%d]属于合并单元格，值为[%s]' % (row_index, col_index, cell_value))
                    return cell_value
                    break
        return None


def format_dict2list(r_dict):
    l_tup = sorted(r_dict.items(), key=lambda item: item[0], reverse=True)
    r_list = []
    max_num = 0
    for k, v in l_tup:
        if v["level"] > max_num:
            max_num = v["level"]
        result = []
        if v["level"] == max_num:
            result.append(v["text"])
            recursion(r_dict, result, v["parent"])
            r_list.append(result[::-1])
    return r_list


def recursion(r_dict, result, id):
    ps = r_dict[id]
    text, parent = ps["text"], ps["parent"]
    result.append(text)
    if parent in r_dict:
        recursion(r_dict, result, parent)


def au_get_tree2dict(filename, sheet_name, level):
    """
    返回数据库标准存储结构
    """
    dict_tool = PandaLabelDictTool()
    label_dict_object = dict_tool.load_tree_dict(
        filename,
        sheet_name=sheet_name,
        header=False,
        tree_format=0,
        total_label_level=level,
        label_name='beauty',
        split_text=False)
    r_dict = label_dict_object.get_dict()
    return r_dict


def format_list2df(result):
    max_len = 0
    for i in result:
        if len(i) > max_len:
            max_len = len(i)

    data = []
    for i in result:
        loss = max_len - len(i)
        tmp_data = i[:-1] + ["" for _ in range(loss)] + [i[-1].split("|")[0], i[-1]]
        data.append(tmp_data)

    max_len += 1
    df = pd.DataFrame(data)
    columns = ["level_%s" % i for i in range(1, max_len + 1)]
    columns[-2] = "keytag"
    columns[-1] = "keyword"
    df.columns = columns
    return df


def au_get_tree(infile, sheet, level, sheet_name):
    r_dict = au_get_tree2dict(infile, sheet, level)
    r_list = format_dict2list(r_dict)
    df = format_list2df(r_list)
    df = df.drop(["keytag"], axis=1)
    df["keyword"] = df["keyword"].apply(lambda line: str(line).replace(" ", "").lower())
    df = df.sort_values(by=list(df.columns), ascending=True)
    columns_dict = au_get_new_columns(level, sheet_name)
    df.rename(columns=columns_dict, inplace=True)
    tree_list = [tree for line, tree in columns_dict.items()]
    return df, tree_list, sheet_name


def au_get_line(path, sheet):
    df = pd.read_excel(path, sheet, error_bad_lines=False, encoding="utf_8_sig")
    line_new_list = []
    line_list = list(df.columns)
    columns_dict = {}
    the_pop1 = 'keyword'  # 后续代码中用的都是keyword，所以我这儿就不改了
    the_pop2 = [i for i in line_list if 'keyword' in i.replace(" ", "").lower()][0]  # 这儿是给的词表里keyword这列的字段名
    for line in line_list:
        new_line = "{}({})".format(sheet, line)
        columns_dict[line] = new_line
        line_new_list.append(new_line)
    new_keyword1 = "{}({})".format(sheet, the_pop1)
    new_keyword2 = "{}({})".format(sheet, the_pop2)
    columns_dict[the_pop1] = new_keyword1
    line_new_list.append(new_keyword1)
    line_new_list.remove(new_keyword2)
    df.rename(columns=columns_dict, inplace=True)
    df[new_keyword1] = df[new_keyword2].apply(lambda line: str(line).replace(" ", "").lower())
    if the_pop2 != the_pop1:
        df.drop(new_keyword2, axis=1, inplace=True)
    return df, line_new_list, sheet


def au_get_new_columns(level, sheet_name):
    columns_dict = collections.OrderedDict()
    level -= 1
    for col in range(level):
        str_col = str(col + 1)
        columns_dict['level_{}'.format(str_col)] = '{}(level{})'.format(sheet_name, str_col)
    columns_dict['keyword'] = "{}(keyword)".format(sheet_name)
    return columns_dict


@contextlib.contextmanager
def pickle_context(filepath, obj, mode='wb'):
    pass


class _MappingCollect:

    def __init__(self, path):
        """
        mapping表的集合,相当于整个标签词表数据集
        :param path:
        """

        df_sheet = pd.read_excel(path, None)
        sheets = list(df_sheet.keys())
        self.__length = len(sheets)
        self._mapping = [Mapping(path, sheet) for sheet in sheets]
        self.relation = False

    def filter_by(self, name):
        """用于查找某一个sheet对象"""
        pass


class MappingCollect(_MappingCollect):
    def filter_by(self, **kwargs):
        """用于查找某一个sheet对象"""
        name = kwargs.get("name")
        filter_mappings = list(filter(lambda mapping: mapping.sheetname == name, self._mapping))
        if len(filter_mappings) > 0:
            filter_mapping = filter_mappings[0]
        else:
            filter_mapping = None
        return filter_mapping

    def add_sen_sheet(self, sheet_list):
        sheet_list = format_max_xml_kwargs(sheet_list)
        if sheet_list:
            for sheet_name in sheet_list:
                sen_result = self.filter_by(name=sheet_name)
                if sen_result is None:
                    print("情感预设失败,标签词表不含{}sheet".format(sheet_name))
                else:
                    sen_result.sentment = True

    def add_length_sheet(self, sheet_dic):
        sheet_dic = format_max_xml_kwargs(sheet_dic)
        if sheet_dic:
            for sheet_name, tag_length in sheet_dic.items():
                sen_result = self.filter_by(name=sheet_name)
                if sen_result is None:
                    print("距离设置失败,标签词表不含{}sheet".format(sheet_name))
                else:
                    sen_result.length = tag_length

    def __getitem__(self, item):
        return self._mapping[item]

    def to_pickle(self, param):
        save_pickle = open(param, 'wb')
        pickle.dump(self, save_pickle)
        save_pickle.close()


class _Mapping:
    """
    对单个mapping表的数据结构进行封装,相当于单一sheet
    """

    def __init__(self, path, sheet):
        tree_line_tup = self.with_draw(path, sheet)
        self.mapping_list = tree_line_tup[1]
        self.mapping_df = tree_line_tup[0]
        self.sheetname = tree_line_tup[-1]
        self.sentment = True
        self.length = 0
        self.relation = None

    def with_draw(self, path, sheet):
        if "-" in sheet:
            sheet_name = sheet.split('-')[0]
            sheet_level = int(sheet.split('-')[-1])
            tree_tup = au_get_tree(path, sheet, sheet_level, sheet_name)
            return tree_tup
        else:
            line_tup = au_get_line(path, sheet)
            return line_tup

    @property
    def mapping_values(self):
        mapping_df = self.mapping_df.astype(str)
        maping_values = mapping_df[self.mapping_list].values
        return maping_values

    @property
    def choice(self):
        return 'content'


class Mapping(_Mapping):

    def __init__(self, path, sheet):
        super(Mapping, self).__init__(path, sheet)
        self.verifi = self.verification()

    def verification(self):
        datapram = {
            'code': 0,
            'msg': "词表正常",
            'sheetname': self.sheetname
        }
        before_verifi_list = self.mapping_values
        verifi_list = [verifi[-1] for verifi in before_verifi_list]
        for veri in verifi_list:
            if str(veri) in ["", "||"]:
                datapram['code'] = -1
                datapram['msg'] = "词表异常"
                # print("异常码:{},异常提示:{},词表名:{}".format(datapram['code'],datapram['msg'],datapram['sheetname']))
                return datapram
            if str(veri)[0] == "|" or str(veri[-1]) == "|":
                datapram['code'] = -2
                datapram['msg'] = "词表异常"
                # print("异常码:{},异常提示:{},词表名:{}".format(datapram['code'], datapram['msg'], datapram['sheetname']))

        return datapram


class _Relation_Mapping_collect(MappingCollect):
    def __init__(self, path):
        super(_Relation_Mapping_collect, self).__init__(path)
        self._mapping = [self.add_primary_id(relation_mapping) for relation_mapping in self._mapping]
        self.relation = True

    def add_primary_id(self, mapping):
        name = mapping.sheetname
        df = mapping.mapping_df
        df['{}-id'.format(name)] = range(df.shape[0])
        mapping.mapping_df = df
        return mapping


class Relation_Mapping_collect(_Relation_Mapping_collect):
    def get_map_name_collect(self):
        map_name_collect = [_name.sheetname for _name in self]
        return map_name_collect


def format_max_xml_kwargs(value):
    value = eval(value) if value else value
    return value


def extract_context(path, keyword):
    mapping = MappingCollect(path)
    data_list = []
    for i in mapping:
        for j in i.mapping_list:
            data = i.mapping_df[i.mapping_df[j] == keyword].values
            if len(data) > 0:
                data_list.append(list(data))
    return data_list


def viewLevel(path, keyword, level_=0):
    data_list = []
    data = extract_context(path, keyword)
    for i in data[0]:
        i = i.tolist()
        data_list.append(i)
    shangji_all_list = []
    xiaji_all_list = []
    for i in data_list:
        if level_:
            if i[level_ - 1] == keyword:
                shangji_all = i[:level_ - 1]
                xiaji_all = i[level_:]
                for j in shangji_all:
                    shangji_x = (i.index(j) + 1, j)
                    shangji_all_list.append(shangji_x)
                for j in xiaji_all:
                    xiaji_x = (i.index(j) + 1, j)
                    xiaji_all_list.append(xiaji_x)
        else:
            shangji_all = i[:i.index(x)]
            xiaji_all = i[i.index(x) + 1:]
            shangji_all_list.extend(shangji_all)
            xiaji_all_list.extend(xiaji_all)

    return list(set(shangji_all_list)), list(set(xiaji_all_list))


if __name__ == '__main__':
    file_path = 'config_tag_mapping.xlsx'  # 文件路径
    x = '美白'  # 要查看的词
    # x = '美白祛斑'
    # level = 0
    level = 4  # 级别，默认为0, 如果传0返回值没有级别
    result = viewLevel(file_path, x, level)
    print('{}    的上级是：'.format(x), result[0])
    print('{}    的下级是：'.format(x), result[1])



# -*- coding:utf-8 -*-
"""
pandas 相关操作
"""
from __future__ import print_function
from __future__ import absolute_import

import pandas as pd
import os
import xlsxwriter
from lavda.util.file_tool import FileTool


class PandaTool(object):
    @staticmethod
    def read_more(input_data, sheet_name=0, header=0, dtype="str"):
        if isinstance(input_data, str):
            df_input = PandaTool.read(input_data, sheet_name=sheet_name, header=header, dtype=dtype)
        else:
            # 默认为DataFrame
            df_input = input_data

        return df_input

    @staticmethod
    def read(input_file_path, sheet_name=0, header=0, dtype="str", file_match_mode="one"):
        """

        :param input_file_path: string，文件名
        :param sheet_name: string，sheet名
        :param header: int，header是否为title
        :param dtype: string or None
        :param file_match_mode: string,文本匹配模式
                    one:完全匹配一个文件名
                    many-sheet:一个文件的多个sheet,合并为一个dataframe
                    many-sheet-list: 不合并
                    many-file:多个文件，多个dataframe合并为一个dataframe, 使用一个input_file_path作为前缀，
                            例如：input_file_path="test.xlx", 无法完全匹配时，test_1.xlsx, test_2.xlsx文件都会被加载使用
                    many-file-list:不合并
        :return:
        """
        # TODO pd.ExcelFile还不知怎么使用dtype
        if file_match_mode == "one":
            df_result = PandaTool.load_one_file(input_file_path, sheet_name, header, dtype)
        elif file_match_mode == "many-sheet":
            # 多sheet模式
            excel_reader = pd.ExcelFile(input_file_path)  # 指定文件
            sheet_names = excel_reader.sheet_names  # 读取文件的所有表单名，得到列表

            df_list = list()
            # TODO 以第一个sheet为基准，只有columns全都一样的才会加载到一起
            first_columns = None
            for i, sheet_name in enumerate(sheet_names):
                if sheet_name.startswith("Sheet"):
                    continue
                df_data = excel_reader.parse(
                    sheet_name=i, header=header)
                if df_data.shape[0] <= 0:
                    continue
                if i == 0:
                    first_columns = set(df_data.columns)
                else:
                    if first_columns is None:
                        continue
                    # TODO 判断列名是否是一样的
                    cur_columns = set(df_data.columns)
                    if len(first_columns) != len(cur_columns):
                        continue
                    same_count = [1 if i_key in first_columns else 0 for i_key in cur_columns]
                    if sum(same_count) != len(first_columns):
                        continue
                df_list.append(df_data)

            # 合并数据
            df_result = pd.concat(df_list, sort=False)

        elif file_match_mode == "many-file":
            # 多文件模式
            file_params_list = FileTool.get_file_path(input_file_path, suffix=[".xlsx", ".csv"])
            df_list = list()
            for file_path in file_params_list:
                df_one = PandaTool.load_one_file(file_path, sheet_name, header, dtype)
                df_list.append(df_one)

            df_result = pd.concat(df_list, sort=False)
        else:
            df_result = None

        return df_result

    @staticmethod
    def load_one_file(input_file_path, sheet_name=0, header=0, dtype="str"):
        parent_file_path, temp_file_name = os.path.split(input_file_path)
        file_name, extension = os.path.splitext(temp_file_name)
        extension = extension.lower()
        if extension.endswith("xlsx") or extension.endswith("xls"):
            df = pd.read_excel(input_file_path, header=header, sheet_name=sheet_name, dtype=dtype)
            # TODO 按字符串类型时，全部全值转为字符串
            if dtype is not None and dtype == "str":
                df.fillna(value="", inplace=True)

        elif extension.endswith("csv"):
            df = pd.read_csv(input_file_path, header=header, encoding='utf_8_sig')
        else:
            raise Exception("unknown file extension" + input_file_path)

        return df

    @staticmethod
    def save(output_file_path, df_content, max_line=1000000, index=False, file_match_mode="many-file"):
        """

        :param output_file_path: 输出文件路径
        :param df_content: DataFrame
        :param max_line: 最大行数据
        :param index: bool
        :param file_match_mode: string,文本匹配模式
                    many-sheet:一个文件的多个sheet
                    many-file:多个文件，使用一个input_file_path作为前缀，
                            例如：input_file_path="test.xlx", 无法完全匹配时，test_1.xlsx, test_2.xlsx文件都会被加载使用
        :return:
        """
        def get_batch_index(length, batch_size):
            index_list = list()
            start = 0
            while start < length:
                end = start + batch_size
                if end > length:
                    end = length
                index_list.append((start, end))
                start = end
            return index_list

        # 保存csv
        if output_file_path.endswith("csv"):
            df_content.to_csv(output_file_path, encoding='utf_8_sig', index=index)
        elif output_file_path.endswith("xlsx"):
            # 保存xlsx
            # TODO xlsx有行数限制，因此大数据文件要拆分
            line_count = df_content.shape[0]
            MAX_BATCH = max_line
            df_result_list = list()
            if line_count > MAX_BATCH:
                batch_list = get_batch_index(line_count, MAX_BATCH)
                parent_file_path, temp_file_name = os.path.split(output_file_path)
                file_name, extension = os.path.splitext(temp_file_name)
                for i, one_batch in enumerate(batch_list):
                    df_sub_content = df_content.iloc[one_batch[0]: one_batch[1]]
                    df_result_list.append(df_sub_content)

                if file_match_mode == "many-sheet":
                    t_df_list = [(str(ti), df_t_one) for ti, df_t_one in enumerate(df_result_list)]
                    PandaTool.save_many(output_file_path, t_df_list)
                else:
                    for i, df_sub_content in enumerate(df_result_list):
                        sub_output_file_path = os.path.join(parent_file_path, file_name + "_" + str(i) + extension)

                        writer = pd.ExcelWriter(sub_output_file_path, engine='xlsxwriter',
                                                options={'strings_to_urls': False})
                        df_sub_content.to_excel(writer, engine='xlsxwriter', index=index)
                        writer.save()

            else:
                writer = pd.ExcelWriter(output_file_path, engine='xlsxwriter',
                                        options={'strings_to_urls': False})
                df_content.to_excel(writer, engine='xlsxwriter', index=index)
                writer.save()
        else:
            raise Exception("unknown file extension" + output_file_path)

    @staticmethod
    def get_column(df):
        real_columns = list()
        cols = df.columns
        for key in cols:
            real_columns.append(str(key))

        return real_columns

    @staticmethod
    def drop_unnamed_column(df):
        real_column = PandaTool.get_column(df)
        select_column = [column for column in real_column if "Unname" not in column]
        df = df[select_column]

        return df

    @staticmethod
    def add_index(df, index_name):
        index_list = list()
        for index, row in df.iterrows():
            index_list.append(index)

        df[index_name] = index_list
        return df

    @staticmethod
    def save_many(output_file_path, df_many_data, index=False):
        """
        注意不能处理超行的问题
        :param output_file_path:
        :param df_many_data:
        :param index:
        :return:
        """
        writer = pd.ExcelWriter(output_file_path,
                                engine='xlsxwriter',
                                options={'strings_to_urls': False})
        if isinstance(df_many_data, (list, tuple)):
            for sheet_name, df in df_many_data:
                df.to_excel(writer, sheet_name=sheet_name, index=index)
        elif isinstance(df_many_data, dict):
            for sheet_name in df_many_data:
                df_many_data[sheet_name].to_excel(writer, sheet_name=sheet_name, index=index)
        else:
            raise Exception("unknown data type")
        writer.save()

    @staticmethod
    def drop_column(df, column_list):
        real_column = PandaTool.get_column(df)
        for column in column_list:
            real_column.remove(column)
        df = df[real_column]

        return df



# -*- coding: utf-8 -*-
# ----------------------------------------------
# purpose : 
# author : LGD
# create_time : 2021/2/22 18:15
# update_time : 2021/2/22 18:15
# copyright : Lavector
# ----------------------------------------------
import os
import xml.etree.ElementTree as ET


class PyConfigParser(object):

    def get_default_config_dir(self):
        cur_cwd = os.getcwd()
        if os.path.exists(u'configs/'):
            return os.path.join(cur_cwd, u'configs/')
        elif os.path.exists(u'../config/'):
            return os.path.join(cur_cwd, u'../configs/')
        elif os.path.exists(u'../../configs/'):
            return os.path.join(cur_cwd, u'../../configs/')
        elif os.path.exists(u'../../../configs/'):
            return os.path.join(cur_cwd, u'../../../configs/')

        raise Exception("can not find confir root path!")

    def _clean_string(self, data):
        return data.strip()

    def parse_xml(self, file_name, file_dir=None):
        """

        :param file_name: 配置文件名，不带后缀
        :param file_dir: 配置文件所在的目录，相对路径和绝对路径均可以，
                         为None时，使用默认路径: ./configs/
        :return:
        """
        if file_dir:
            file_path = os.path.join(file_dir, file_name + u'.xml')
        else:
            default_dir = self.get_default_config_dir()
            file_path = os.path.join(default_dir, file_name + u'.xml')

        tree = ET.parse(file_path)
        root_node = tree.getroot()
        config_dict = dict()

        for section_child in root_node:
            tag_name = section_child.attrib[u'name']
            tag_name = self._clean_string(tag_name)
            config_dict[tag_name] = dict()
            if len(section_child) <= 0:
                continue

            for item_node in section_child:
                item_key = self._clean_string(item_node.attrib[u'name'])
                item_value = self._clean_string(item_node.text)
                config_dict[tag_name][item_key] = item_value

        return config_dict


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @Author : dada
# @time   : 2020/7/13 14:18
import pandas as pd
from lavda.analysis.da_tag.Filed_Services import filed_services
from lavda.analysis.da_tag.tag import related_word_mapping,get_Text2Formal,TextMatch2,index_list
from lavda.analysis.da_tag.tag_config import tag_choice_dic



def max_split_tag(df,mappings,Intermediate,add_head_text):
    return last_max_split_tag(df=df,mappings=mappings,Intermediate=Intermediate,add_head_text=add_head_text)

def max_relation_split_tag(df,mappings,intermediate,add_head_text):
    return last_max_relation_split_tag(df=df,mappings=mappings,intermediate=intermediate,add_head_text=add_head_text)



@filed_services
def last_max_split_tag(df, mappings, Intermediate, add_head_text):
    for _map in mappings:
        df['{}_other'.format(_map.sheetname)] = df[Intermediate].apply(lambda line: eval(line)['{}'.format(_map.sheetname)] if line and str(line) != "nan" else "")
    for mapping in mappings:
        df['{}_index'.format(mapping.sheetname)] = range(df.shape[0])
        keyword = mapping.mapping_list
        Sen = mapping.sentment
        tag_other_list = df[['{}_index'.format(mapping.sheetname),'{}_other'.format(mapping.sheetname)]].values.tolist()
        tag_list = []
        for tag in tag_other_list:
            index = tag[0]
            G_tag = str(tag[1]).split("噜")
            tag_list += [[index, G_id] for G_id in G_tag]

        df_tag = pd.DataFrame(tag_list, columns=["{}_index".format(mapping.sheetname), "{}_other".format(mapping.sheetname)])
        df = df.drop(['{}_other'.format(mapping.sheetname)], axis=1,)
        df = pd.merge(df, df_tag, on="{}_index".format(mapping.sheetname), how="left")
        for word in enumerate(keyword):
            df[word[1]] = df['{}_other'.format(mapping.sheetname)].apply(lambda line: str(line).split("*")[word[0]] if len(line) > 0 else "" )
        if Sen:
            df["{}_aspect".format(mapping.sheetname)] = df['{}_other'.format(mapping.sheetname)].apply(lambda line: str(line).split("*")[-1] if len(line) > 0 else "")
        df = df.drop(['{}_other'.format(mapping.sheetname), '{}_index'.format(mapping.sheetname)], axis=1)
    df = df.drop([Intermediate], axis=1)
    df.drop_duplicates(keep="first", inplace=True)
    return df


@filed_services
def last_max_relation_split_tag(df,mappings,intermediate,add_head_text):
    df['merge_index'] = range(df.shape[0])
    keyword_list = []  # 存放keyword列的字段名，下面提取打到的词会用到

    map_collect_list = mappings.get_map_name_collect()
    if  len(map_collect_list) == 1:
        collet_name = map_collect_list[0]
        df['tag_example'] = df[intermediate].apply(lambda line:signle_mapping_deal(line,collet_name))
        tag_other_list = df[
            ['merge_index', 'tag_example']].values.tolist()
        tag_list = []
        for tag in tag_other_list:
            index = tag[0]
            G_tag = tag[1]
            tag_list += [[index, G_id] for G_id in G_tag]

        df_tag = pd.DataFrame(tag_list,
                              columns=["merge_index", "{}-id".format(collet_name)])

        df1 = pd.merge(df, df_tag, on="merge_index", how="left")
        df2 = pd.merge(df1,mappings[0].mapping_df , on="{}-id".format(collet_name), how="left")
        df2 = df2.drop(['{}-id'.format(collet_name),'merge_index','tag_example','tag_keyword', intermediate], axis=1, )
        for i in df2:
            if '(keyword)' in i and add_head_text not in i:
                keyword_list.append(i)
        for i in keyword_list:
            df2[i.split('(')[0] + '_aspect'] = df2.apply(lambda line: add_aspect(line, i), axis=1)
        df2 = df2.drop(['tag_aspect'], axis=1)
        return df2
    else:
        split_merge_index = df.groupby('merge_index')
        df_list = [related_word_mapping(intermediate,the_data, mappings) for name, the_data in split_merge_index]
        df_data = pd.concat(df_list,sort=False)
        df_data.drop(['merge_index', 'tag_keyword', intermediate], axis=1, inplace=True)
        for i in df_data:
            if '(keyword)' in i and add_head_text not in i:
                keyword_list.append(i)
        for i in keyword_list:
            df_data[i.split('(')[0] + '_aspect'] = df_data.apply(lambda line: add_aspect(line, i), axis=1)
        df_data = df_data.drop(['tag_aspect'], axis=1)
        return df_data


def add_aspect(line, i):
    #TODO 提取aspect词
    '''
    先这么用，之后会改，写的太草了
    为分句打标增加实际打上的字段（情感使用）
    :param line:传进来的一列数据
    :param i: 传进来的keyword列字段名，比如：时间(keyword)
    :return:
    '''
    result_list = []  # 存放结果
    keywords_list = []  # 存放结果
    keywords_list2 = []  # 存放结果
    keywords_list3 = []  # 存放结果
    if '|' in str(line[i]):
        for split_1 in str(line[i]).split('|'):
            if '-' in split_1:
                keywords_list.append(split_1.split('-')[0])
            else:
                keywords_list.append(split_1)
    else:
        if '-' in str(line[i]):
            keywords_list.append(str(line[i]).split('-')[0])
        else:
            keywords_list.append(str(line[i]))
    for i in keywords_list:
        if '(' in i or '（' in i:
            keywords_list2.append(i[1:-1])
        else:
            keywords_list2.append(i)
    for i in keywords_list2:
        if '+' in i:
            keywords_list3.extend(i.split('+'))
        else:
            keywords_list3.append(i)

    if line['tag_aspect']:
        for i in eval(line['tag_aspect']):
            if '#' in i and i in keywords_list3:
                result_list.append(i[1:])
            elif '@' in i and i in keywords_list3:
                text = str(line['content'])+str(line['title'])
                text_list = [text[tag_index-5 if tag_index-5>=0 else 0:tag_index+len(i[1:])+5] for tag_index in index_list(text,i[1:])]
                for _ in text_list:
                    for foudingci in TextMatch2.negative_word_list:
                        if foudingci in _:
                            result_list.extend([foudingci,i[1:]])
                            break
                    break
            elif i in keywords_list3:
            # if i in keywords_list3:
                result_list.append(i)
    return ','.join(set(result_list))


def signle_mapping_deal(line,collectname):
    if str(line) in ["nan", "", "NaN", "null", "Null", "None"]:
        result = [-1]
    else:
        line = eval(line)
        if len(line) == 1:
            if str(line[0]) ==  '({})'.format(collectname):
                result = [-1]
            else:
                result = [int(str(line[0]).replace('({})'.format(collectname),''))]
        else:
            result = [int(str(tag_).replace('({})'.format(collectname),'')) for tag_ in line]
    return result


# -*- coding:utf-8 -*-
"""
apriori 算法
"""


class Apriori(object):
    def __init__(self, print_detail=False):
        self.print_detail = print_detail

    def createC1(self, dataSet):
        C1 = []
        for transaction in dataSet:
            for item in transaction:
                if not [item] in C1:
                    C1.append([item]) #store all the item unrepeatly

        C1.sort()
        #return map(frozenset, C1)#frozen set, user can't change it.
        return list(map(frozenset, C1))

    def scanD(self, D,Ck,minSupport):
        """
        
        :param D: 数据集
        :param Ck: 候选项集列表Ck
        :param minSupport: 感兴趣项集的最小支持度 minSupport
        :return: 
        """
        ssCnt={}
        for tid in D:#遍历数据集
            for can in Ck:#遍历候选项
                if can.issubset(tid):#判断候选项中是否含数据集的各项
                    #if not ssCnt.has_key(can): # python3 can not support
                    if not can in ssCnt:
                        ssCnt[can]=1 #不含设为1
                    else: ssCnt[can]+=1#有则计数加1
        numItems=float(len(D))#数据集大小
        retList = []#L1初始化
        supportData = {}#记录候选项中各个数据的支持度
        for key in ssCnt:
            support = ssCnt[key]/numItems#计算支持度
            if support >= minSupport:
                retList.insert(0, key)#满足条件加入L1中
                supportData[key] = support
        return retList, supportData

    def aprioriGen(self, Lk, k):
        """
        #组合，向上合并
        :param Lk: 频繁项集列表 Lk
        :param k: 项集元素个数 k
        :return: 
        """
        retList = []
        lenLk = len(Lk)
        for i in range(lenLk):
            for j in range(i+1, lenLk): #两两组合遍历
                L1 = list(Lk[i])[:k-2]; L2 = list(Lk[j])[:k-2]
                L1.sort(); L2.sort()
                if L1==L2: #若两个集合的前k-2个项相同时,则将两个集合合并
                    retList.append(Lk[i] | Lk[j]) #set union
        return retList

    def process(self, dataSet, minSupport = 0.5):
        """
        
        :param dataSet: list(list()), 输入数据
        :param minSupport: float，最小支持度
        :return: 
        """
        C1 = self.createC1(dataSet)
        D = list(map(set, dataSet)) #python3
        L1, supportData = self.scanD(D, C1, minSupport)#单项最小支持度判断 0.5，生成L1
        L = [L1]
        k = 2
        while (len(L[k-2]) > 0):#创建包含更大项集的更大列表,直到下一个大的项集为空
            Ck = self.aprioriGen(L[k-2], k)#Ck
            Lk, supK = self.scanD(D, Ck, minSupport)#get Lk
            supportData.update(supK)
            L.append(Lk)
            k += 1
        return L, supportData

    def generateRules(self, L, supportData, minConf=0.7):
        #频繁项集列表、包含那些频繁项集支持数据的字典、最小可信度阈值
        bigRuleList = [] #存储所有的关联规则
        for i in range(1, len(L)):  #只获取有两个或者更多集合的项目，从1,即第二个元素开始，L[0]是单个元素的
            # 两个及以上的才可能有关联一说，单个元素的项集不存在关联问题
            for freqSet in L[i]:
                H1 = [frozenset([item]) for item in freqSet]
                #该函数遍历L中的每一个频繁项集并对每个频繁项集创建只包含单个元素集合的列表H1
                if (i > 1):
                #如果频繁项集元素数目超过2,那么会考虑对它做进一步的合并
                    self.rulesFromConseq(freqSet, H1, supportData, bigRuleList, minConf)
                else:#第一层时，后件数为1
                    self.calcConf(freqSet, H1, supportData, bigRuleList, minConf)# 调用函数2
        return bigRuleList

    def calcConf(self, freqSet, H, supportData, brl, minConf=0.7):
        # 针对项集中只有两个元素时，计算可信度
        # 返回一个满足最小可信度要求的规则列表
        prunedH = []
        for conseq in H:
            # 后件，遍历 H中的所有项集并计算它们的可信度值
            # 可信度计算，结合支持度数据
            if freqSet not in supportData or freqSet-conseq not in supportData:
                # TODO 一些数据可能不在支持度里面，这些数据一律不要了
                continue
            conf = supportData[freqSet]/supportData[freqSet-conseq]
            if conf >= minConf:
                if self.print_detail:
                    print(freqSet-conseq, '-->', conseq, 'conf:', conf)
                # 如果某条规则满足最小可信度值,那么将这些规则输出到屏幕显示
                # 添加到规则里，brl 是前面通过检查的 bigRuleList
                brl.append((freqSet-conseq, conseq, conf))
                # 同样需要放入列表到后面检查
                prunedH.append(conseq)
        return prunedH

    def rulesFromConseq(self, freqSet, H, supportData, brl, minConf=0.7):
        #参数:一个是频繁项集,另一个是可以出现在规则右部的元素列表 H
        m = len(H[0])
        if (len(freqSet) > (m + 1)): #频繁项集元素数目大于单个集合的元素数
            Hmp1 = self.aprioriGen(H, m+1)#存在不同顺序、元素相同的集合，合并具有相同部分的集合
            Hmp1 = self.calcConf(freqSet, Hmp1, supportData, brl, minConf)#计算可信度
            if (len(Hmp1) > 1):
            #满足最小可信度要求的规则列表多于1,则递归来判断是否可以进一步组合这些规则
                self.rulesFromConseq(freqSet, Hmp1, supportData, brl, minConf)


if __name__ == '__main__':

    dataSet = [['豆奶', '苹果'],
               ['豆奶', '莴苣'],
               ['莴苣', '尿布', '葡萄酒', '甜菜'],
               ['豆奶', '尿布', '葡萄酒', '橙汁'],
               ['莴苣', '豆奶', '尿布', '葡萄酒'],
               ['莴苣', '豆奶', '尿布', '橙汁']]

    processor = Apriori()
    L, suppData = processor.process(dataSet, 0.5)

    rules = processor.generateRules(L, suppData, 0.6)
    print(L)
    print("----")
    print(suppData)
    print("-----")
    print(rules)



# -*- coding: utf-8 -*-
# ----------------------------------------------
# author : lgd
# create_time : 2020/6/29 14:04
# update_time : 2020/6/29 14:04
# file : age_format.py
# copyright : Lavector
# ----------------------------------------------
"""
年龄相关处理
"""
import datetime

from lavda.backexcutor.analysis_interface import RowInterface
from lavda.analysis.da_extract.datetime_format import DateTimeFormat


class AgeLevelFormat(RowInterface):
    def __init__(self, default_value=None):
        """
        :param default_value: int, 无数据或非法数据时返回此默认值
        """
        super(AgeLevelFormat, self).__init__(default_value)
        self.default_value = default_value
        self.age_level_list = {'41-50': 7,
                               '31-40': 5,
                               '31-35': 5,
                               '21-30': 3,
                               '21-25': 3,
                               '11-20': 2,
                               }

    def process(self, data, params=None):
        if data is None:
            return self.default_value
        age = str(data).strip()
        if not age:
            return self.default_value
        # 0615新加的规则 ,从简介里面捞取年龄
        if '-' in age:
            return self.age_level_list[age]
        age = int(float(age))
        if age <= 0:
            return self.default_value

        age = int(age)
        if 0 < age < 10:
            return self.default_value
        if 10 <= age <= 15:
            return self.default_value
        if 16 <= age <= 20:
            return 2
        if 21 <= age <= 25:
            return 3
        if 26 <= age <= 30:
            return 4
        if 31 <= age <= 35:
            return 5
        if 36 <= age <= 40:
            return 6
        if 41 <= age <= 45:
            return 7
        if 46 <= age <= 50:
            return 8
        if 51 <= age <= 55:
            return 9
        if 56 <= age <= 60:
            return 10

        return self.default_value


class CalAgeFromBirth(RowInterface):
    def __init__(self, default_value=None):
        """
        从生日时间中提出年龄信息
        :param default_value: 无年龄数据或数据无效时使用此值
        """
        super(CalAgeFromBirth, self).__init__(default_value)
        self.default_value = default_value

    def process(self, data, **params):

        """
        根据时间文本字符串计算年龄
        :param data: datetime or string(时间格式字符串，"%Y-%m-%d %H:%M:%S")
        :param params:
        :return: int, 年龄
        """
        if data is None:
            return self.default_value
        # 数据格式检验
        if isinstance(data, str):
            if data.count("-") != 2 or data.count(":") != 2 or len(data) < 18:
                return self.default_value
            data = datetime.datetime.strptime(data, "%Y-%m-%d %H:%M:%S")

        birth_year = data.year

        if birth_year is None:
            return None

        # 计算年龄
        now_year = datetime.datetime.now().year
        age = now_year - birth_year + 1

        return age


class AgeFormat(RowInterface):
    """
    提取并格式化年龄数据，将不合法的数据使用默认值进行表示
    支持的数据包括两种：1. 数字类 2.年龄格式的字符串
    """

    def __init__(self, default_value=None, min_age=16, max_age=60):
        """
        格式化时间数据
        :param default_value: 无年龄数据或数据无效时使用此值
        :param min_age: int or None, 有效数据最小值, 包含等于 <=
        :param max_age: int or None, 有效数据最大值，包含等于  >=
        """
        super(AgeFormat, self).__init__(default_value)

        self.default_value = default_value
        self.min_age = min_age
        self.max_age = max_age

        self.time_extract_processor = DateTimeFormat()
        self.cal_age_processor = CalAgeFromBirth()

    def process(self, data, **params):
        """

        :param data: int or string，输入数据，整数或日期格式数据(作为初生日期)
        :param params:
        :return:
        """
        if 'age' not in data:
            return self.default_value
        data_age = data['age']
        if data_age == '':
            return self.supplementary_age(data)

        if isinstance(data_age, int):
            age = data_age
        else:
            age_str = str(data_age).strip()
            if age_str.replace('.', '', 1).isdigit():
                age = int(float(age_str))
            else:
                # 按年龄处理
                try:
                    datetime_string = self.time_extract_processor.parse(age_str)
                    age = self.cal_age_processor.process(datetime_string)
                except Exception as e:
                    age = self.default_value

        if age is None:
            return self.default_value

        # 年龄数据校验
        if self.min_age is not None and age < self.min_age:
            return self.default_value
        if self.max_age is not None and age > self.max_age:
            return self.default_value

        return age

    def supplementary_age(self, data, **params):
        '''
        0615新加的规则 ,从简介里面捞取年龄
        如果年龄为空，通过简介，用户标签判断年龄信息
        :param data: 字典类型一行数据
        :param params:
        :return:
        '''
        age_list = {'70后': '41-50',
                    '80后': '31-40',
                    '85后': '31-35',
                    '90后': '21-30',
                    '95后': '21-25',
                    '00后': '11-20',
                    '70後': '41-50',
                    '80後': '31-40',
                    '85後': '31-35',
                    '90後': '21-30',
                    '95後': '21-25',
                    '00後': '11-20',
                    }
        judge_column_name_list = ['introduce', 'user_label', 'nickname']
        for column_name in judge_column_name_list:
            if column_name not in data:
                data[column_name] = ''
        judge_text = '.'.join([str(data[i]) for i in judge_column_name_list if data[i]]).lower()
        for _age in age_list.keys():
            if _age in judge_text:
                return age_list[_age]
        return self.default_value


if __name__ == '__main__':
    age_level = AgeLevelFormat()
    print(age_level.process(19))

    age_format = AgeFormat()
    print(age_format.process("60"))



# -*- coding: utf-8 -*-
# ----------------------------------------------
# purpose : 
# author : 
# create_time : 2020/7/1 16:46
# update_time : 2020/7/1 16:46
# copyright : Lavector
# ----------------------------------------------
import re


def indexMany(The_Str,The_Find):   #str是要查询的字符
    length = len(The_Str)     #获取该字符串的长度
    str1 = The_Str            #拷贝字符串
    The_List = []
    count = 0             #用来计算每次截取完字符串的总长度
    try:
        while str1.index(The_Find)!=-1:      #当字符串中没有该字符则跳出
            n = str1.index(The_Find)         #查询查找字符的索引
            str2 = str1[0:n + 1]        #截取的前半部分
            str1 = str1[n + 1:length]   #截取的后半部分
            count = count + len(str2)       #计算每次截取完字符串的总长度
            The_List.append(count - 1)        #把所有索引添加到列表中
            length=length-len(str2)     #截取后半部分的长度
    except ValueError:
        return The_List
    return The_List


def add_self_length(content,tag):
    result_list = indexMany(content, tag)


    return_list = [result+len(tag) for result in result_list]
    return return_list



def continuous_addition(content,tag,preset_length):
    assert type(preset_length) == int , "长度不符合规范"
    tag_list = tag.split("+")
    assert len(tag_list) == 2, "关键词长度不符合规范"
    if not re.search("(?=.*{})(?=.*{})^.*$".format(tag_list[0], tag_list[1]), content):
        return False
    first_word = tag_list[0]
    end_word = tag_list[1]
    first_length = len(first_word)
    second_length = len(end_word)
    first_index = indexMany(content,first_word)
    second_index = indexMany(content,end_word)
    for first in first_index:
        for second in second_index:
            if first < second:
                length = second-(first+first_length)
            else:
                length = first-(second+second_length)
            if length < preset_length:
                return True

    return False



# -*- coding: utf-8 -*-
# ----------------------------------------------
# author : 
# create_time : 2020/6/29 15:36
# update_time : 2020/6/29 15:36
# copyright : Lavector
# ----------------------------------------------
from pymysql import *
import pandas as pd
import copy

from lavda.util.py_config_parser import PyConfigParser
from lavda.config.maxda_config import *
from sqlalchemy import create_engine


class LavwikiReader(object):
    """
    加载 lavwiki 数据库的数据
    """

    @staticmethod
    def get_connection_config(dict_format=False):
        # TODO 注意要复制一份而不是直接使用，因此下一面还要修改值的
        db_config = copy.deepcopy(CONF_DB_LAVWIKI)

        if dict_format:
            db_config['cursorclass'] = cursors.DictCursor

        return db_config

    @staticmethod
    def get_as_df(db_table_name, dict_format=True):
        # 注意没有列名的
        # 创建连接
        config = LavwikiReader.get_connection_config(dict_format)
        con = connect(**config)
        # 执行sql语句
        try:
            with con.cursor() as cursor:
                sql = "select * from %s" % db_table_name
                cursor.execute(sql)
                result = cursor.fetchall()
        finally:
            con.close()
        df = pd.DataFrame(result)
        return df

    @staticmethod
    def execute(sql, dict_format=False):
        db = None
        cur = None
        try:
            config = LavwikiReader.get_connection_config(dict_format)
            db = connect(**config)
            # 游标对象
            cur = db.cursor()
            cur.execute("%s;" % sql)
            db.commit()
            msg = "success"
        except Exception as e:
            # 错误回滚
            db.rollback()
            msg = "fail"
        finally:
            cur.close()
            db.close()
        # 返回统一状态
        return msg

    # 数据库查询所有操作方法:
    @staticmethod
    def execute_all(sql, dict_format=False):
        db = None
        cur = None
        import traceback
        try:
            config = LavwikiReader.get_connection_config(dict_format)
            db = connect(**config)
            cur = db.cursor()
            cur.execute("%s;" % sql)
            result = cur.fetchall()
            msg = "success"
        except Exception as e:
            # 错误回滚
            db.rollback()
            msg = "fail"
            result = []
        finally:
            cur.close()
            db.close()
        # 返回统一状态
        return result, msg


class LavwikiDfReader(object):
    @staticmethod
    def read_table(table_name, database="lavwiki"):
        """

        :param table_name: string,表名
        :param database: string，数据库名
        :return:
        """
        db_config = CONF_DB_LAVWIKI
        db_config['database'] = database
        conn = connect(**db_config)
        sql = """select * from {0}{1}""".format("", table_name)
        df = pd.read_sql(sql, conn)
        conn.close()
        return df


class CommonLavDbReader(object):
    """
    加载 lav 数据库的数据
    """

    @staticmethod
    def get_connection_config(dict_format=False, database='lav_ai', default_db_config=None):
        # TODO 注意要复制一份而不是直接使用，因此下一面还要修改值的
        if default_db_config is None:
            parser = PyConfigParser()
            config = parser.parse_xml("configs")
            db_config = config['lav_db']
        else:
            db_config = default_db_config.copy()
        db_config['port'] = int(db_config['port'])
        db_config['database'] = database

        if dict_format:
            db_config['cursorclass'] = cursors.DictCursor

        return db_config

    @staticmethod
    def get_as_df(db_table_name, dict_format=True, database='lav_ai', db_config=None):
        """

        :param db_table_name: string, 表名
        :param dict_format: bool, 是否返回dict格式的数据
        :param database: string，数据库名
        :param db_config: dict, 数据库连接配置
        :return:
        """
        # 注意没有列名的
        # 创建连接
        config = CommonLavDbReader.get_connection_config(dict_format, database=database, default_db_config=db_config)
        con = connect(**config)
        # 执行sql语句
        try:
            with con.cursor() as cursor:
                sql = "select * from %s" % db_table_name
                cursor.execute(sql)
                result = cursor.fetchall()
        finally:
            con.close()
        df = pd.DataFrame(result)
        return df

    @staticmethod
    def execute(sql, dict_format=False, database='lav_ai', db_config=None):
        db = None
        cur = None
        try:
            config = CommonLavDbReader.get_connection_config(dict_format, database, default_db_config=db_config)
            db = connect(**config)
            # 游标对象
            cur = db.cursor()
            cur.execute("%s;" % sql)
            db.commit()
            msg = "success"
        except Exception as e:
            # 错误回滚
            db.rollback()
            msg = "fail"
        finally:
            cur.close()
            db.close()
        # 返回统一状态
        return msg

    # 数据库查询所有操作方法:
    @staticmethod
    def execute_all(sql, dict_format=False, database='lav_ai', db_config=None):
        db = None
        cur = None
        try:
            config = CommonLavDbReader.get_connection_config(dict_format, database, default_db_config=db_config)
            db = connect(**config)
            cur = db.cursor()
            cur.execute("%s;" % sql)
            result = cur.fetchall()
            msg = "success"
        except Exception as e:
            # 错误回滚
            db.rollback()
            msg = "fail"
            result = []
        finally:
            cur.close()
            db.close()
        # 返回统一状态
        return result, msg


# 增加个线上数据库的engine
cloud_server_engine = create_engine(
    'mysql+pymysql://data_analysis:data_analysis_root_0715@111.207.114.228:13306/lavtest')
local_server_engine = create_engine(
    'mysql+pymysql://data_analysis:data_analysis_root_0715@111.207.114.228:13306/lavtest')

if __name__ == '__main__':
    config = LavwikiReader.get_connection_config()
    LavwikiReader.execute_all("select keyword from ec_des_word")



# -*- coding: utf-8 -*-
# ----------------------------------------------
# purpose: 其他类垃圾检测
# author : 
# create_time : 2020/6/29 15:49
# update_time : 2020/6/29 15:49
# copyright : Lavector
# ----------------------------------------------
import re

from lavda.analysis.da_clean.clean_interface import *


class NoChineseEnglish(RowInterface, CleanInterface):
    """
    非中英文过滤
    """
    def __init__(self, **params):
        super(NoChineseEnglish, self).__init__(CleanFlag.NO)

    def process(self, data, **params):
        zhPattern = re.compile(u'[\u4e00-\u9fa5]+')
        match1 = zhPattern.search(str(data))
        match2 = re.findall('[a-zA-Z]+', str(data))
        if match1 or len(match2) > 0:
            return CleanFlag.NO
        return CleanFlag.YES


class TextLen(RowInterface, CleanInterface):
    def __init__(self, **params):
        super(TextLen, self).__init__(CleanFlag.NO)

    def process(self, data, **params):
        if len(str(data).strip()) < 2:
            return CleanFlag.YES
        # TODO 原来只是超短文本过滤，20210816将微博超话过滤加入进去
        # 微博超话过滤规则：content 以超话宝石符号开头，整体小于15
        #  宝石符号
        if '' == data.strip()[0] and len(str(data).strip()) <= 15:
            return CleanFlag.YES
        return CleanFlag.NO


if __name__ == '__main__':
    no_ce = NoChineseEnglish()
    print(no_ce.process("234###"))

    text_len = TextLen()
    print(text_len.process("khhh"))


# -*- coding: utf-8 -*-
# ----------------------------------------------
# purpose: 电商类垃圾检测
# author : 
# create_time : 2020/6/29 15:46
# update_time : 2020/6/29 15:46
# copyright : Lavector
# ----------------------------------------------
import numpy as np

from lavda.analysis.da_clean.clean_interface import *
from lavda.backexcutor.analysis_interface import RowInterface
from lavda.util.mysql_util import *


class EcSpam(RowInterface, CleanInterface):
    """
    电商垃圾的处理
    """
    def __init__(self, **params):
        super(EcSpam, self).__init__(CleanFlag.NO)

        # 加载数据
        sql = """
        select keyword from %s
        """ % "ec_des_word"
        result = np.array(LavwikiReader.execute_all(sql))[0]
        result = [item[0] for item in result]
        clean = [str(i).lower().strip() for i in result if len(str(i).strip()) > 0]

        self.clean_list = clean

    def process(self, data, **params):
        """
        电商垃圾的标记
        """
        data = str(data).replace(" ", "").lower()
        for i in self.clean_list:
            if i in data:
                return CleanFlag.YES
        return CleanFlag.NO


if __name__ == '__main__':
    spam = EcSpam()
    print(spam.process("救积分"))
    print(spam.process("你好呀"))


# -*- coding: utf-8 -*-
# ----------------------------------------------
# purpose : 关键词规则匹配
# author : LGD
# create_time : 2021/2/2 09:57
# update_time : 2021/2/2 09:57
# copyright : Lavector
# ----------------------------------------------
import ahocorasick
from typing import List, Optional


class KeywordMark(object):
    """
    keyword rule中定义的特殊标记
    """
    # TODO 加减号可以一起使用，加减号没有方向，是双向的
    # 1个加号限制距离为7个字符，>1个加号则没有距离限制
    # 减号没有距离限制
    PLUS = "+"
    MINUS = "-"
    # 否定词标记可以出现在任何词前面，但也只能用在词前面，不能是词的组合，如小括号等
    # 否定词距离为5，没有方向限制
    NEG_YES = "@"  # 要否定词
    NEG_NO = "#"   # 不要否定词

    AND = "|"
    # 括号前面必须是加号、减号或空（即开头或结尾）
    BRACKET_LEFT = "("
    BRACKET_RIGHT = ")"


class KElem(object):
    def __init__(self, need_neg):
        self._need_neg = need_neg

    @property
    def need_neg(self):
        return self._need_neg


class KItem(KElem):
    def __init__(self, k_str):
        if k_str.startswith(KeywordMark.NEG_YES):
            k_str = k_str.replace(KeywordMark.NEG_YES, "")
            self._neg_mark = KeywordMark.NEG_YES
        elif k_str.startswith(KeywordMark.NEG_NO):
            k_str = k_str.replace(KeywordMark.NEG_NO, "")
            self._neg_mark = KeywordMark.NEG_NO
        else:
            self._neg_mark = None
        self._k_str = k_str
        super(KItem, self).__init__(True if self._neg_mark is not None else False)

    @property
    def k_str(self):
        return self._k_str

    @property
    def neg_mark(self):
        return self._neg_mark

    @property
    def to_string(self):
        return self._k_str


class KRule(KElem):
    def __init__(self, master, p_slave, m_slave, rule_str=None):
        """

        :param master: string or KRule， 主索引词
        :param p_slave: list(KItem or KRule), 条件词
        :param m_slave: list(KItem or KRule), 条件词
        :param rule_str: string, 规则字符串

        """
        if KeywordMark.NEG_YES in rule_str or KeywordMark.NEG_NO in rule_str:
            need_neg = True
        else:
            need_neg = False
        super(KRule, self).__init__(need_neg)

        self._master = master
        self._p_slave = p_slave
        self._m_slave = m_slave

        # 规则所对应的keyword id
        self._k_id = None

        # 规则id
        self._rule_id = None

        self._rule_str = rule_str

    @property
    def to_string(self):
        return self._rule_str

    @property
    def rule_str(self):
        return self._rule_str

    @property
    def rule_id(self):
        return self._rule_id

    @rule_id.setter
    def rule_id(self, value):
        self._rule_id = value

    @property
    def k_id(self):
        return self._k_id

    @k_id.setter
    def k_id(self, value):
        self._k_id = value

    def get_master_words(self):
        """
        获取master 的所有词
        :return:
        """
        return KRule.get_word(self._master)

    def get_slave_words(self):
        w_list = list()
        for item in self._p_slave:
            t_list = KRule.get_word(item)
            w_list.extend(t_list)
        for item in self._m_slave:
            t_list = KRule.get_word(item)
            w_list.extend(t_list)
        return w_list

    @staticmethod
    def get_word(rule_item):
        """

        :param rule_item: KItem or KRule
        :return:
        """
        if isinstance(rule_item, KItem):
            return [rule_item.k_str]
        else:
            # 由于只有两层构造，因此所有节点都是KItem
            w_list = list()
            w_list.append(rule_item.master.k_str)
            for item in rule_item.p_slave:
                w_list.extend(KRule.get_word(item))
            for item in rule_item.m_slave:
                w_list.extend(KRule.get_word(item))

            return w_list

    @property
    def master(self):
        return self._master

    @property
    def p_slave(self):
        return self._p_slave

    @property
    def m_slave(self):
        return self._m_slave


class NegWordFinder(object):
    def __init__(self):
        neg_word = ['不', "过于", '没有', '未必', '木有', '未能', '从未', '无法', '不会', '不怎么']
        neg_auto = ahocorasick.Automaton()
        for word in neg_word:
            neg_auto.add_word(word, word)
            neg_auto.make_automaton()
        self.neg_auto = neg_auto

    def find(self, content):
        """

        :param content:
        :return: 返回否定词位置索引信息
        """
        neg_pos_set = set()
        for item in self.neg_auto.iter(content):
            target_last_index = item[0]
            target_word = item[-1]
            end = target_last_index + 1
            start = end - len(target_word)
            pos_list = list(range(start, end))
            neg_pos_set.update(pos_list)

        return neg_pos_set


class KRuleParser(object):
    """
    keyword 规则解析器
    """
    # 匹配方向
    DIRECTION_RIGHT = 'right'
    DIRECTION_LEFT = "left"
    DIRECTION_BOTH = "both"

    EXCLUDE_CH = {",", '，', '。',  ':', '：', '！', '?', '？'}

    def __init__(self, kid2keyword: dict, build_neg_model=False):
        """

        :param kid2keyword: dict, id -> keyword 字符串
                        eg: {1: "(中国+完成)-(重要+事情)-知道吗|中华"}
        """
        self.kid2keyword = kid2keyword
        # 1.解析规则
        kid2rules = {k_id: None for k_id in kid2keyword}
        rule_id_dict = dict()

        for k_id in kid2keyword:
            rule_list = self.parse_rule_from_keyword(kid2keyword[k_id])
            for i, cur_rule in enumerate(rule_list):
                cur_rule.k_id = k_id
                cur_rule.rule_id = str(k_id) + "-" + str(i+1)
                rule_id_dict[cur_rule.rule_id] = cur_rule
            kid2rules[k_id] = rule_list

        # 2.获取所有的词
        # master词和slave词分开创建也以减少不需要的查找，因为多数标签只有master words或没有master words
        all_master_words = list()
        all_slave_words = list()
        mater_word_rule_dict = dict()
        for k_id in kid2rules:
            k_rules = kid2rules[k_id]
            for one_rule in k_rules:
                master_words = one_rule.get_master_words()
                all_master_words.extend(master_words)

                slave_words = one_rule.get_slave_words()
                all_slave_words.extend(slave_words)
                # TODO 暂时只取master第一个词作为索引词
                if len(master_words) > 1:
                    all_slave_words.extend(master_words[1:])
                master_first_word = master_words[0]
                if master_first_word not in mater_word_rule_dict:
                    mater_word_rule_dict[master_first_word] = set()
                mater_word_rule_dict[master_first_word].add(one_rule.rule_id)

        # 3.构建词典存储结构
        master_auto = ahocorasick.Automaton()
        for word in mater_word_rule_dict:
            word_rules = [rule_id_dict[r_id] for r_id in mater_word_rule_dict[word]]
            master_auto.add_word(word, (word, word_rules))
        master_auto.make_automaton()
        self.master_auto = master_auto

        slave_auto = ahocorasick.Automaton()
        for word in all_slave_words:
            slave_auto.add_word(word, word)
            slave_auto.make_automaton()
        self.slave_auto = slave_auto

        # 4.否定词查找模型
        if build_neg_model:
            self.neg_model = NegWordFinder()
        else:
            self.neg_model = None

    def parse(self, content,
              return_all=False,
              neg_pos_set=None,
              direction="both",
              plus_max_len=7,
              neg_max_len=5,
              minus_max_range=7,
              merge_keyword=True):
        """

        :param content:
        :param return_all: bool， 是否返回所有匹配结果
        :param neg_pos_set: set, 否定词位置序列
        :param direction: string, 搜索匹配方向
        :param plus_max_len: int, 加号控制距离
        :param neg_max_len: int, 否定词控制距离
        :param minus_max_range: int, 减号作用范围，双向控制，分别向目标词左、右各扩展 minux_max_range 的内容
        :param merge_keyword: bool, 是否合并keyword result，因为一个keyword一个位置附近可能匹配到多个asect词
                                True:合并这些keyword，取最长的
                                False：不合并
        :return:
            eg: [{"aspect": "中国", 'start': 0, 'end': 3, 'keyword': 中国, 'kid': 1}]
        """
        if len(content) <= 0:
            return None

        # 1.先查找索引词
        master_search_list = list()
        slave_word_list = list()
        master_search_word_list = list()
        need_neg_flag = False
        for item in self.master_auto.iter(content):
            target_last_index = item[0]
            target_word, target_rules = item[-1]
            end = target_last_index + 1
            start = end - len(target_word)
            master_search_list.append((start, end, target_word, target_rules))
            master_search_word_list.append((start, end, target_word))

            for one_rule in target_rules:
                if not need_neg_flag and one_rule.need_neg:
                    need_neg_flag = True
                slave_words = one_rule.get_slave_words()
                slave_word_list.extend(slave_words)
                master_words = one_rule.get_master_words()
                # TODO 暂时只取第一个词以外的词，第一个词已经作为索引词了
                if len(master_words) > 1:
                    slave_word_list.extend(master_words[1:])
        # 去重
        slave_word_list = list(set(slave_word_list))

        # 2.查找限定词/条件词
        slave_search_word_list = list()
        if len(slave_word_list) > 0:
            # 查找slave word
            tmp_slave_auto = ahocorasick.Automaton()
            for word in slave_word_list:
                tmp_slave_auto.add_word(word, word)
                tmp_slave_auto.make_automaton()
            slave_search_word_list = self.search_from_automaton(tmp_slave_auto, content)
        search_word_list = master_search_word_list + slave_search_word_list

        # 3.生成总的 word->位置 词典
        word_pos_dict = dict()
        for start, end, word in search_word_list:
            if word not in word_pos_dict:
                word_pos_dict[word] = set()
            word_pos_dict[word].add((start, end))

        # 排序
        for word in word_pos_dict:
            # 排序，从左往右
            sort_result = sorted(list(word_pos_dict[word]), key=lambda x: x[0])
            word_pos_dict[word] = sort_result

        # 4.检查是否需要查看否定词
        if need_neg_flag:
            if neg_pos_set is None:
                if self.neg_model is not None:
                    neg_pos_set = self.neg_model.find(content)
        if neg_pos_set is None:
            neg_pos_set = set()

        # 4.匹配
        kid_result_dict = dict()
        for start, end, target_word, target_rules in master_search_list:
            # TODO 不同的rule可能有相同的rule_str，因此需要减少不必要的计算量
            cache_rule_str_dict = dict()
            for one_rule in target_rules:
                kid = one_rule.k_id
                # 不需要查找所有时，只要匹配到一个词即可返回
                if not return_all and kid in kid_result_dict:
                    continue
                cur_rule_str = one_rule.rule_str
                if cur_rule_str in cache_rule_str_dict:
                    one_result = cache_rule_str_dict[cur_rule_str]
                else:
                    if direction == self.DIRECTION_BOTH:
                        # TODO,双向匹配时，优先使用向右匹配，无法命中时，再向左匹配
                        one_result = self.match(content, target_word, start, end, one_rule, word_pos_dict, neg_pos_set, self.DIRECTION_RIGHT, plus_max_len, neg_max_len, minus_max_range)
                        if one_result is None:
                            one_result = self.match(content, target_word, start, end, one_rule, word_pos_dict, neg_pos_set,
                                                    self.DIRECTION_LEFT, plus_max_len, neg_max_len, minus_max_range)

                    else:
                        one_result = self.match(content,target_word, start, end, one_rule, word_pos_dict, neg_pos_set, direction, plus_max_len, neg_max_len, minus_max_range)
                    cache_rule_str_dict[cur_rule_str] = one_result
                if one_result is None:
                    continue

                if kid not in kid_result_dict:
                    kid_result_dict[kid] = list()
                kid_result_dict[kid].append(one_result)

        # 5.消歧（一个Keyword的多个规则可能匹配到了多个词，因此要消歧,将一个keyword的所有数据汇总消歧）
        k_word_result = list()
        for kid in kid_result_dict:
            k_keyword = self.kid2keyword[kid]
            k_pos_list = kid_result_dict[kid]
            if len(k_pos_list) == 1:
                start, end = k_pos_list[0]
                target_content = content[start: end]
                item = {"aspect": target_content, 'start': start, 'end': end, 'keyword': k_keyword, 'kid': kid}
                k_word_result.append(item)
            else:
                # 以最后一们作为消歧依据，挑选最长的匹配
                if merge_keyword:
                    end_dict = dict()
                    for start, end in k_pos_list:
                        if end not in end_dict:
                            end_dict[end] = set()
                        end_dict[end].add(start)

                    for end in end_dict:
                        s_list = end_dict[end]
                        start = min(list(s_list))
                        target_content = content[start: end]
                        item = {"aspect": target_content, 'start': start, 'end': end, 'keyword': k_keyword, 'kid': kid}
                        k_word_result.append(item)
                else:
                    for start, end in k_pos_list:
                        target_content = content[start: end]
                        item = {"aspect": target_content, 'start': start, 'end': end, 'keyword': k_keyword, 'kid': kid}
                        k_word_result.append(item)

        return k_word_result

    def match(self, content: str,
              word: str,
              start: int,
              end: int,
              one_rule: KRule,
              word_pos_dict: dict,
              neg_pos_set: set,
              direction: str,
              plus_max_len: int,
              neg_max_len: int,
              minus_max_range: int) -> tuple:
        """
        :param content:string, 原始内容
        :param word:string,索引词
        :param start:int,索引词起始位置
        :param end:int,索引词起始位置
        :param one_rule:KRule, 目标规则
        :param word_pos_dict:dict, 所有查找词位置信息
        :param neg_pos_set:set, 否定词位置信息
        :param direction:string, 搜索匹配方向
        :param plus_max_len:int, 搜索匹配方向
        :param neg_max_len:int, 搜索匹配方向
        :return:tuple(start, end), 一条规则匹配到的目标词的起始位置信息
        """
        # 1. 检查减号（无方向、无距离(小括号还是有距离的))
        # if self.is_exist_minus_items(one_rule.m_slave, word_pos_dict, neg_pos_set, plus_max_len=plus_max_len, neg_max_len=neg_max_len):
        #     return None

        # 2.检查master 索引词
        select_word_list = list()
        first_word = {"word": word, "start": start, "end": end}
        select_word_list.append(first_word)

        # print(first_word, one_rule.rule_str)

        master = one_rule.master
        # 检查并判断master的索引词
        if isinstance(master, KItem):
            if not self.is_match_word_list([(start, end)], neg_pos_set, master.neg_mark, neg_max_len):
                return None
        else:
            first_kitem = master.master
            if not self.is_match_word_list([(start, end)], neg_pos_set, first_kitem.neg_mark, neg_max_len):
                return None

        # 3. 查找匹配其他词，如其他加号词，master索引词之外的词
        left_result = None
        right_result = None
        if direction == self.DIRECTION_RIGHT or direction == self.DIRECTION_BOTH:
            right_result = self.search_one_direction(start, end, one_rule, word_pos_dict, neg_pos_set, self.DIRECTION_RIGHT, plus_max_len, neg_max_len)

        if direction == self.DIRECTION_LEFT or direction == self.DIRECTION_BOTH:
            left_result = self.search_one_direction(start, end, one_rule, word_pos_dict, neg_pos_set, self.DIRECTION_LEFT, plus_max_len, neg_max_len)

        # TODO 结果可以为空，但不能为None
        if right_result is None and left_result is None:
            return None
        if right_result is not None:
            select_word_list.extend(right_result)
        if left_result is not None:
            select_word_list.extend(left_result)

        # 4.合并所有词找到最大组合范围
        target_pos_list = list()
        for item in select_word_list:
            target_pos_list.append(item['start'])
            target_pos_list.append(item['end'])

        target_start = min(target_pos_list)
        target_end = max(target_pos_list)

        # 5.减号距离判断
        if len(one_rule.m_slave) > 0:
            # m_left = target_start - minus_max_range
            # m_right = target_end + minus_max_range
            m_word_pos_dict = dict()
            for k_word in word_pos_dict:
                # TODO 构建减号词作用域范围，一般是候选目标词的一小段前后扩展
                if minus_max_range is not None:
                    m_value = [item for item in word_pos_dict[k_word] if self.get_pos_dis(target_start, target_end, item[0], item[1]) <= minus_max_range]
                else:
                    m_value = [item for item in word_pos_dict[k_word]]
                m_word_pos_dict[k_word] = m_value

            if self.is_exist_minus_items(one_rule.m_slave, m_word_pos_dict, neg_pos_set, plus_max_len=plus_max_len, neg_max_len=neg_max_len):
                return None

        # 6.过虑检查
        match_word = content[target_start: target_end]
        if self.is_filter(match_word):
            return None

        return target_start, target_end

    def is_filter(self, match_word):
        for ch in list(match_word):
            if ch in self.EXCLUDE_CH:
                return True
        # 中英点号特殊判断,有些英文字符就是使用点号分开的，如人名之类的，因此要排除的，不能一概作为标点符号进行处理
        return False

    def search_one_direction(self,
                             start: int,
                             end: int,
                             one_rule: KRule,
                             word_pos_dict: dict,
                             neg_pos_set: set,
                             direction: str,
                             plus_max_len: int,
                             neg_max_len: int):
        """
        根据指定的方向查找/匹配一个完整的规则模式
        :param start:
        :param end:
        :param one_rule:
        :param word_pos_dict:
        :param neg_pos_set:
        :param direction:
        :param plus_max_len:
        :param neg_max_len:
        :return:
        """

        # 1.master 为组合词时需要查找另一个词
        master = one_rule.master
        select_word_list = list()
        if isinstance(master, KRule):
            second_kitem = master.p_slave[0]
            # master 是括号组合关系
            other_master_word = second_kitem.k_str
            near_word_result = self.get_near_word(start,
                                                  end,
                                                  other_master_word,
                                                  second_kitem.neg_mark,
                                                  neg_pos_set,
                                                  word_pos_dict,
                                                  direction,
                                                  neg_max_len=neg_max_len,
                                                  max_len=plus_max_len)
            if near_word_result is None:
                return None
            select_word_list.append(near_word_result)

        # 2.检查加号词
        long_max_len = plus_max_len
        for item in one_rule.p_slave:
            if len(select_word_list) > 0:
                # 选用上一个词作为起始查找位置
                source_start = select_word_list[-1]['start']
                source_end = select_word_list[-1]['end']
            else:
                source_start = start
                source_end = end

            if isinstance(item, KItem):
                target_word_item = self.get_near_word(source_start, source_end, item.k_str, item.neg_mark, neg_pos_set, word_pos_dict, direction, neg_max_len=neg_max_len, max_len=long_max_len)
                if target_word_item is None:
                    return None
                select_word_list.append(target_word_item)
            else:
                i_kitem1 = item.master
                i_kitem2 = item.p_slave[0]

                group_result = self.get_near_group(source_start,
                                                   source_end,
                                                   i_kitem1.k_str,
                                                   i_kitem1.neg_mark,
                                                   i_kitem2.k_str,
                                                   i_kitem2.neg_mark,
                                                   neg_pos_set,
                                                   word_pos_dict,
                                                   direction=direction,
                                                   plus_max_len=plus_max_len,
                                                   neg_max_len=neg_max_len,
                                                   max_len=long_max_len)
                if group_result is None:
                    return None
                select_word_list.extend(group_result)

        return select_word_list

    def get_near_word(self, source_start: int,
                      source_end: int,
                      other_word: str,
                      other_word_neg_mark,
                      neg_pos_set,
                      word_pos_dict: dict,
                      direction="right",
                      neg_max_len=None,
                      max_len=None) -> dict:
        """
        根据源词位置找到最近的目标词，可能有方向和距离限制
        :param source_start:
        :param source_end:
        :param other_word:
        :param other_word_neg_mark:
        :param neg_pos_set:
        :param word_pos_dict:
        :param direction:
        :param neg_max_len:
        :param max_len:
        :return: dict
                eg: {"start": 11, "end": 13, 'word': '中国'}
        """
        if other_word not in word_pos_dict:
            return None

        # 3.找词并作距离判断
        may_pos_list = word_pos_dict[other_word]
        target = None
        if direction == self.DIRECTION_RIGHT:
            tmp_list = [item for item in may_pos_list if item[0] > source_start]
            if len(tmp_list) <= 0:
                return None
            # 选第一个
            for one_target in tmp_list:
                if max_len is not None:
                    dis = self.get_pos_dis(source_start, source_end, one_target[0], one_target[1])
                    if dis > max_len:
                        return None
                if not self.is_match_word_list([(one_target[0], one_target[1])], neg_pos_set, other_word_neg_mark, neg_max_len):
                    continue

                target = one_target
                break

        elif direction == self.DIRECTION_LEFT:
            tmp_list = [item for item in may_pos_list if item[0] < source_start]
            if len(tmp_list) <= 0:
                return None
            # 反转，从最后一个开始匹配
            tmp_list.reverse()
            for one_target in tmp_list:
                if max_len is not None:
                    dis = self.get_pos_dis(source_start, source_end, one_target[0], one_target[1])
                    if dis > max_len:
                        return None
                if not self.is_match_word_list([(one_target[0], one_target[1])], neg_pos_set, other_word_neg_mark, neg_max_len):
                    continue

                target = one_target
                break
        else:
            raise Exception("unsupport directionr for get_near_word")

        if target is None:
            return None

        item_result = {"start": target[0], "end": target[1], 'word': other_word}

        return item_result

    def get_near_group(self, source_start: int,
                       source_end: int,
                       word1: str,
                       word1_neg_mark: KeywordMark,
                       word2: str,
                       word2_neg_mark: KeywordMark,
                       neg_pos_set: set,
                       word_pos_dict: dict,
                       direction="right",
                       plus_max_len=None,
                       neg_max_len=None,
                       max_len=None):
        """
        找到距离 source_start, source_end最近的两个词
        TODO 只支持向左或向右查找
        :param source_start:
        :param source_end:
        :param word1:
        :param word1_neg_mark:
        :param word2:
        :param word2_neg_mark:
        :param word_pos_dict:
        :param neg_pos_set:
        :param direction:
        :param plus_max_len:
        :param neg_max_len:
        :param max_len:
        :return:
        """
        # TODO word_pos_dict 中的位置信息必须是排序好的
        if word1 not in word_pos_dict:
            return None
        if word2 not in word_pos_dict:
            return None

        if direction == self.DIRECTION_RIGHT:
            word1_pos_list = [item for item in word_pos_dict[word1] if item[0] >= source_start]

            for s1, e1 in word1_pos_list:
                # 检查否定词语
                if not self.is_match_word_list([(s1, e1)], neg_pos_set, word1_neg_mark, neg_max_len):
                    continue

                # 检查距离
                if max_len is not None:
                    dis = self.get_pos_dis(s1, e1, source_start, source_end)
                    if dis > max_len:
                        return None

                # 括号组内查找总是使用距离限制的
                target2_item = self.get_near_word(s1, e1, word2, word2_neg_mark, neg_pos_set, word_pos_dict, direction=direction, max_len=plus_max_len, neg_max_len=neg_max_len)
                if target2_item is None:
                    return None

                item1 = {"word": word1, "start": s1, 'end': e1}
                return item1, target2_item
        elif direction == self.DIRECTION_LEFT:
            word1_pos_list = [item for item in word_pos_dict[word1] if item[0] < source_start]
            # 要先反转，从最后一个开始向前查找
            word1_pos_list.reverse()
            for s1, e1 in word1_pos_list:
                # 检查否定词语
                if not self.is_match_word_list([(s1, e1)], neg_pos_set, word1_neg_mark, neg_max_len):
                    continue

                # 括号组内查找总是使用距离限制的
                target2_item = self.get_near_word(s1, e1, word2, word2_neg_mark, neg_pos_set, word_pos_dict, direction=direction, max_len=plus_max_len, neg_max_len=neg_max_len)
                if target2_item is None:
                    return None

                # 检查距离
                if max_len is not None:
                    dis = self.get_pos_dis(target2_item['start'], target2_item['end'], source_start, source_end)
                    if dis > max_len:
                        return None

                item1 = {"word": word1, "start": s1, 'end': e1}
                return item1, target2_item
        else:
            raise Exception("unsupport directionr for get_near_group")

        return None

    def get_pos_dis(self, start1, end1, start2, end2) -> int:
        """
        两个词的距离，有重叠时距离为0
        :param start1:
        :param end1:
        :param start2:
        :param end2:
        :return:
        """
        # TODO 这里的两词间距离，实际上是两个两个词间隔字符的数量
        # TODO 一个词的 end位置，其实是该词后面的一位，因此两前后两词的end和start位置相减，正是它们的间隔距离
        if start1 <= start2:
            first = end1
            last = start2
        else:
            first = end2
            last = start1

        # TODO 距离是指两个词的间隔
        dis = max(0, last-first+1)

        return dis

    def is_match_word_list(self, word_pos_list, neg_pos_set, neg_mark=None, neg_max_len=0):
        """
        TODO 匹配词语，否定词判断有问题的，因为可能多个word，有的前面有否定词，有的没有否定词
        :param word_pos_list: [(1, 2), (5, 9)]
        :param neg_pos_set: [3,4,5]
        :param neg_mark:
        :param neg_max_len:
        :return:
        """
        if len(word_pos_list) <= 0:
            return False
        if neg_mark is None:
            return True

        # 检查否定词
        has_neg = False
        for start, end in word_pos_list:
            # TODO 双向匹配
            target_pos = list(range(start-neg_max_len, start)) + list(range(end, end + neg_max_len))

            for i in target_pos:
                if i in neg_pos_set:
                    has_neg = True
                    break
            if has_neg:
                break

        if has_neg:
            if neg_mark == KeywordMark.NEG_YES:
                return True
            else:
                return False
        else:
            if neg_mark == KeywordMark.NEG_YES:
                return False
            else:
                return True

    def is_exist_minus_items(self, m_slave: list,
                             word_pos_dict: dict,
                             neg_pos_set: set,
                             plus_max_len: int,
                             neg_max_len: int):
        """
        检查 减号标记的词是否存在，存在则规则匹配失败
        :param m_slave:
        :param word_pos_dict:
        :param neg_pos_set:
        :param plus_max_len:
        :param neg_max_len:
        :return:
        """
        for item in m_slave:
            if isinstance(item, KItem):
                # TODO 存在即不失败
                if item.k_str in word_pos_dict and self.is_match_word_list(word_pos_dict[item.k_str], neg_pos_set, item.neg_mark, neg_max_len):
                    return True
            else:
                # 组合括号关系，应该只有两个词
                master_word = item.master.k_str
                slave_word = item.p_slave[0].k_str
                group_result = self.is_exist_group(master_word,
                                                   item.master.neg_mark,
                                                   slave_word,
                                                   item.p_slave[0].neg_mark,
                                                   word_pos_dict,
                                                   neg_pos_set,
                                                   neg_max_len,
                                                   plus_max_len)
                if group_result:
                    return True
        return False

    def is_exist_group(self, word1,
                       word1_neg_mark,
                       word2,
                       word2_neg_mark,
                       word_pos_dict,
                       neg_pos_set,
                       neg_max_len,
                       max_len=None) -> bool:
        """
        检查是否存在两个组合词 （小括号包起来的词）,不区分方向，找到一个即可以
        :param word1:
        :param word1_neg_mark:
        :param word2:
        :param word2_neg_mark:
        :param word_pos_dict:
        :param neg_pos_set:
        :param neg_max_len:
        :param max_len:
        :return:
        """
        # 词语是否存在匹配
        if word1 not in word_pos_dict:
            return False
        if word2 not in word_pos_dict:
            return False

        w1_pos_list = word_pos_dict[word1]
        w2_pos_list = word_pos_dict[word2]

        # 距离匹配
        if max_len is None:
            return True
        for s1, e1 in w1_pos_list:
            for s2, e2 in w2_pos_list:
                dis = self.get_pos_dis(s1, e1, s2, e2)

                neg1_result = self.is_match_word_list([(s1, e1)], neg_pos_set, word1_neg_mark, neg_max_len)
                neg2_result = self.is_match_word_list([(s2, e2)], neg_pos_set, word2_neg_mark, neg_max_len)
                if not neg1_result or not neg2_result:
                    continue

                if dis <= max_len:
                    return True

        return False

    def search_from_automaton(self, auto_object, content):
        search_list = list()
        for item in auto_object.iter(content):
            target_last_index = item[0]
            target_word = item[1]
            end = target_last_index + 1
            start = end - len(target_word)
            search_list.append((start, end, target_word))

        return search_list

    @staticmethod
    def second_split(input_str: str) -> list:
        """
        拆分一条规则字符串
        :param input_str: string
                eg: "中国+完成-(重要+事情)-知道吗"
        :return: list
                eg:['中国', '+完成', '-(重要+事情)', '-知道吗']
        """
        sub_list = list()
        tmp_list = list()
        input_str_list = list(input_str)
        i = 0
        while i < len(input_str_list):
            ch = input_str_list[i]
            if ch == KeywordMark.BRACKET_LEFT:
                j = i + 1
                while j < len(input_str_list) and input_str_list[j] != KeywordMark.BRACKET_RIGHT:
                    j += 1

                tmp_list.extend(list(input_str_list[i:j+1]))
                tmp_sub_str = "".join(tmp_list)

                # 递归处理,只有一层递归，不要太复杂了
                sub_list.append(tmp_sub_str)
                # print(input_str_list[j+1])
                i = j + 1
                tmp_list = list()

                continue
            if ch in {KeywordMark.PLUS, KeywordMark.MINUS}:
                if len(tmp_list) > 0:
                    sub_list.append("".join(tmp_list))
                tmp_list = list()
                # 分隔符放在第一位
                tmp_list.append(ch)
            else:
                tmp_list.append(ch)

            i += 1

        if len(tmp_list) > 0:
            sub_list.append("".join(tmp_list))

        return sub_list

    @staticmethod
    def build_rule_item(sub_str: str) -> KRule:
        """
        构建一条规则
        :param sub_str: string, 规则字符串
        :return:
     """
        def build_simple_k_item(k_one_str):
            if k_one_str.startswith(KeywordMark.PLUS):
                k_one_str = k_one_str.replace(KeywordMark.PLUS, "")
                return KeywordMark.PLUS, KItem(k_one_str)
            elif k_one_str.startswith(KeywordMark.MINUS):
                k_one_str = k_one_str.replace(KeywordMark.MINUS, "")
                return KeywordMark.MINUS, KItem(k_one_str)
            else:
                return None, KItem(k_one_str)

        def build_simple_k_rule(k_one_str):
            if k_one_str.startswith(KeywordMark.PLUS):
                k_one_str = k_one_str[len(KeywordMark.PLUS):]
                k_one_str = k_one_str.replace(KeywordMark.BRACKET_LEFT, "")
                k_one_str = k_one_str.replace(KeywordMark.BRACKET_RIGHT, "")
                cur_rule = KRuleParser.build_rule_item(k_one_str)
                return KeywordMark.PLUS,  cur_rule
            elif k_one_str.startswith(KeywordMark.MINUS):
                k_one_str = k_one_str[len(KeywordMark.MINUS):]
                k_one_str = k_one_str.replace(KeywordMark.BRACKET_LEFT, "")
                k_one_str = k_one_str.replace(KeywordMark.BRACKET_RIGHT, "")
                cur_rule = KRuleParser.build_rule_item(k_one_str)
                return KeywordMark.MINUS, cur_rule
            else:
                k_one_str = k_one_str.replace(KeywordMark.BRACKET_LEFT, "")
                k_one_str = k_one_str.replace(KeywordMark.BRACKET_RIGHT, "")
                cur_rule = KRuleParser.build_rule_item(k_one_str)
                return None, cur_rule

        master = None

        rule_str_list = KRuleParser.second_split(sub_str)
        slave_dict = {KeywordMark.PLUS: list(),
                      KeywordMark.MINUS: list()}

        for i, one_str in enumerate(rule_str_list):
            if i == 0:
                if KeywordMark.BRACKET_LEFT in one_str:
                    _, master = build_simple_k_rule(one_str)
                else:
                    _, master = build_simple_k_item(one_str)

            else:
                if KeywordMark.BRACKET_LEFT in one_str:
                    item_type, item = build_simple_k_rule(one_str)
                else:
                    item_type, item = build_simple_k_item(one_str)
                slave_dict[item_type].append(item)

        k_rule = KRule(master, slave_dict[KeywordMark.PLUS], slave_dict[KeywordMark.MINUS], rule_str=sub_str)

        return k_rule

    @staticmethod
    def parse_rule_from_keyword(keyword_str: str) -> List[KRule]:
        """

        :param keyword_str: string, 关键词规则
                    eg: "中国+完成-(重要+事情)-知道吗"|名片夹|卡片夹|护照夹
        :return: list(KRule)
        """
        # 一级拆分
        t_list = keyword_str.split(KeywordMark.AND)

        rule_list = list()
        for sub_str in t_list:
            one_rule = KRuleParser.build_rule_item(sub_str)
            rule_list.append(one_rule)

        return rule_list


if __name__ == "__main__":
    keyword = "(中国+完成)+(想+要)-(重要+事情)-知道吗|#中华"
    # split_result = KRuleParser.second_split(data)
    # print(split_result)
    # rules = KRuleParser.parse_rule_from_keyword(data)
    #
    # for one_rule in rules:
    #     print(one_rule.get_master_words(), one_rule.get_slave_words())
    content = "中国已经完成了最想要12345678知道吗事情，我们还要继续努力人不中华必人胜"
    # content = "我们还要继续努力不中华必胜"
    kid2keyword = {1: keyword}
    parser = KRuleParser(kid2keyword, build_neg_model=True)

    result = parser.parse(content, return_all=True, direction="right")
    print(result)
    # print(KRuleParser.second_split("想+要"))





